package com.gic.cloud.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.IndexDTO;
import com.gic.cloud.dto.IndexDescDTO;

import java.util.List;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-23 14:50
 */
public interface IndexDescApiService {

    /**
     * 查询某个商户在某个模块下的指标
     * @Title: listByEnterpriseAndModule
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param moduleId
     * @param indexGroupName
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<List<IndexDescDTO>> listByEnterpriseAndModule(Integer enterpriseId, String moduleId, String indexGroupName);

    /**
     * 查询
     * @Title: listByEnterpriseAndModuleDetail
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param moduleId
     * @param indexGroupName
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.cloud.dto.IndexEnterpriseGroupDTO>
     * @throws
     */
    ServiceResponse<List<IndexDescDTO>> listByEnterpriseAndModuleDetail(Integer enterpriseId, String moduleId, String indexGroupName, Integer userId);

    /**
     * 回收站中的指标
     * @Title: listRecycle
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param moduleId
     * @param indexGroupName
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.cloud.dto.IndexEnterpriseGroupDTO>
     * @throws
     */
    ServiceResponse<List<IndexDescDTO>> listRecycle(Integer enterpriseId, String moduleId, String indexGroupName, Integer userId);

    /**
     * 排序
     * @Title: sortIndex
     * @Description:
     * @author zhiwj
     * @param enterpriseId 商户id
     * @param moduleId
     * @param indexGroupName
     * @param indexDescEnterpriseId 指标id
     * @param seq 排到第几个
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> sort(Integer enterpriseId, String moduleId, String indexGroupName, Integer indexDescEnterpriseId, Integer seq);

    /**
     * 移动到回收站
     * @Title: moveToRecycle
     * @Description:
     * @author zhiwj
     * @param indexDescEnterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> moveToRecycle(Integer indexDescEnterpriseId);

    /**
     * 查询
     * @Title: getIndexDesc
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param moduleId
     * @param indexGroupName
     * @param indexCode
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.cloud.dto.IndexDescDTO>
     * @throws
     */
    ServiceResponse<IndexDTO> getIndexDesc(Integer enterpriseId, String moduleId, String indexGroupName, String indexCode);

    /**
     * 批量更新
     * @Title: batchUpdate
     * @Description:
     * @author zhiwj
     * @param descList
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> batchUpdate(List<IndexDescDTO> descList);

    /**
     * 批量更新
     * @Title: batchUpdate
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param moduleId
     * @param ids
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> batchUpdate(Integer enterpriseId, String moduleId, String ids);

    /**
     * 置灰
     * @Title: eraseUpdateTips
     * @Description:
     * @author zhiwj
     * @param userId
     * @param updateTipsId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> eraseUpdateTips(Integer userId, Integer updateTipsId);

    /**
     * 查询全部
     * @Title: listAll
     * @Description:
     * @author zhiwj
     * @param moduleIds 
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.cloud.dto.IndexDescDTO>>
     * @throws
     */
    ServiceResponse<List<IndexDescDTO>> listAllUnRel(String moduleIds);

    /**
     * 初始化
     * @Title: init 
     * @Description:
     * @author zhiwj
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> init(Integer enterpriseId);
}
