package com.gic.cloud.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.qo.UserQo;

import java.util.List;
/**
 * 用户接口
 * @ClassName: UserApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 11:09 AM 
 */
public interface UserApiService {

    /**
     * 新增
     * @Title: saveUser 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveUser(UserDTO dto);

    /**
     * 编辑
     * @Title: editUser 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> editUser(UserDTO dto);

    /**
     * 删除用户
     * @Title: deleteUser 
     * @Description:
     * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> deleteUser(Integer userId);

    /**
     * 删除账号分组下的所有用户
     * @Title: deleteUserByAccountGroupid 
     * @Description:
     * @author guojuxing
     * @param accountGroupId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> deleteUserByAccountGroupId(Integer accountGroupId);

    /**
     * 分页查询用户列表
     * @Title: pageUser 
     * @Description:
     * @author guojuxing
     * @param userQo 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.cloud.dto.UserDTO>> 
     */
    ServiceResponse<Page<UserDTO>> pageUser(UserQo userQo);

    /**
     * 用户列表数据
     * @Title: listUser 
     * @Description:
     * @author guojuxing
     * @param userQo 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.cloud.dto.UserDTO>> 
     */
    ServiceResponse<List<UserDTO>> listUser(UserQo userQo);

    /**
     * 批量转移分组
     * @Title: bulkTransferAccountGroup 
     * @Description:
     * @author guojuxing
     * @param userIdList
     * @param targetAccountGroupId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> bulkTransferAccountGroup(List<Integer> userIdList, Integer targetAccountGroupId);

    /**
     * 手机号查询用户
     * @Title: listUserByPhone 
     * @Description:
     * @author guojuxing
     * @param nationCode 手机区号
     * @param phone 手机号
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.cloud.dto.UserDTO>> 
     */
    ServiceResponse<List<UserDTO>> listUserByPhone(String nationCode, String phone);

    /**
     * 获取用户
     * @Title: getUser 
     * @Description:
     * @author guojuxing
     * @param userId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.cloud.dto.UserDTO> 
     */
    ServiceResponse<UserDTO> getUser(Integer userId);
}
