package com.gic.cloud.operation.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.authcenter.security.core.util.UserUtils;
import com.gic.cloud.dto.DataExplainDTO;
import com.gic.cloud.dto.IndexLogDTO;
import com.gic.cloud.qo.DataExplainQo;
import com.gic.cloud.service.DataExplainApiService;
import com.gic.cloud.service.IndexLogApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-09 17:20
 */
@RestController
public class DataExplainController {

    @Autowired
    private DataExplainApiService dataExplainApiService;
    @Autowired
    private IndexLogApiService indexLogApiService;

    @RequestMapping("/save-data-explain")
    public RestResponse saveDataExplain(DataExplainDTO dataExplainDTO) {
        if (StringUtils.isBlank(dataExplainDTO.getDataName()) || StringUtils.isBlank(dataExplainDTO.getIndexRemark())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
        }
        Integer id = UserUtils.getUser().getId();
        String realName = UserUtils.getUser().getRealName();
        dataExplainDTO.setOptUserId(id);
        dataExplainDTO.setOptUserName(realName);
        ServiceResponse<Void> serviceResponse = dataExplainApiService.saveDataExplain(dataExplainDTO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-data-explain")
    public RestResponse updateDataExplain(DataExplainDTO dataExplainDTO, Integer updateType, String reason) {
        Integer id = UserUtils.getUser().getId();
        String realName = UserUtils.getUser().getRealName();
        dataExplainDTO.setOptUserId(id);
        dataExplainDTO.setOptUserName(realName);
        ServiceResponse<Void> serviceResponse = dataExplainApiService.updateDataExplain(dataExplainDTO, updateType, reason);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-by-data-explain-id")
    public RestResponse getByDataExplainId(Integer dataExplainId) {
        ServiceResponse<DataExplainDTO> serviceResponse = dataExplainApiService.getByDataExplainId(dataExplainId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-data-explain")
    public RestResponse listDataExplain(DataExplainQo dataExplainQo) {
        ServiceResponse<Page<DataExplainDTO>> serviceResponse = dataExplainApiService.listDataExplain(dataExplainQo);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/data-explain-log")
    public RestResponse dataExplainLog(Integer dataExplainId) {
        ServiceResponse<List<IndexLogDTO>> serviceResponse = indexLogApiService.listDataExplainLog(dataExplainId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/delete-data-explain")
    public RestResponse delete(Integer dataExplainId) {
        ServiceResponse<Void> serviceResponse = dataExplainApiService.delete(dataExplainId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/publish-data-explain")
    public RestResponse publishIndex(String dataExplainIds, Integer updateType, String reason) {
        if (StringUtils.isBlank(dataExplainIds)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
        }
        List<Integer> dataExplainIdList;

        String allCode = "all";
        if (allCode.equals(dataExplainIds)) {
            List<DataExplainDTO> explainDTOList = this.dataExplainApiService.listAllDataExplain().getResult();
            dataExplainIdList = explainDTOList.stream().map(DataExplainDTO::getDataExplainId).collect(Collectors.toList());
        } else {
            dataExplainIdList = Stream.of(dataExplainIds.split(",")).map(Integer::valueOf).collect(Collectors.toList());
        }
        ServiceResponse<Void> serviceResponse = dataExplainApiService.publish(dataExplainIdList, updateType, reason, UserUtils.getUser().getId(), UserUtils.getUser().getRealName());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/data-module-list")
    public RestResponse moduleList(String moduleId) {
        Object cache = RedisUtil.getCache("enterprise:datacloud:moduleList");
        if (cache != null) {
            List<JSONObject> resultList = JSON.parseArray(cache.toString(), JSONObject.class);
            Page<DataExplainDTO> page = dataExplainApiService.listDataExplain(new DataExplainQo()).getResult();
            if (page != null) {
                List<DataExplainDTO> result = page.getResult();
                Set<String> usedList = result.stream().filter(e -> StringUtils.isNotBlank(e.getModuleId())).map(DataExplainDTO::getModuleId).collect(Collectors.toSet());

                resultList = resultList.stream().filter(e -> !usedList.contains(e.getString("moduleId")) || StringUtils.equals(e.getString("moduleId"), moduleId)).collect(Collectors.toList());
            }
            return RestResponse.success(resultList);

        }
        return RestResponse.success();
    }

}