package com.gic.cloud.dao.mapper;

import com.gic.cloud.dto.IndexDescDTO;
import com.gic.cloud.entity.TabIndexDesc;
import org.apache.ibatis.annotations.Param;

import java.util.List;
/**
 * 指标
 * @ClassName: TabIndexDescMapper 
 * @Description:  
 * @author zhiwj 
 * @date 2020/9/7 10:45 AM 
 */
public interface TabIndexDescMapper {


    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabIndexDesc record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabIndexDesc record);

    /**
     * 根据主键查询
     *
     * @param indexDescId 主键
     * @return 实体对象
     */
    TabIndexDesc selectByPrimaryKey(Integer indexDescId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabIndexDesc record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabIndexDesc record);

    /**
     * 查询指标
     * @Title: listByEnterpriseAndModule 
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param moduleId
     * @param indexGroupName 
     * @return  java.util.List<com.gic.cloud.dto.IndexDescDTO> 
     */
    List<IndexDescDTO> listByEnterpriseAndModule(@Param("enterpriseId") Integer enterpriseId, @Param("moduleId") String moduleId, @Param("indexGroupName") String indexGroupName);

    /**
     * 查询指标
     * @Title: listByEnterpriseAndModuleDetail 
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param moduleId
     * @param indexGroupName
     * @param userId
     * @param indexStatus 
     * @return  java.util.List<com.gic.cloud.dto.IndexDescDTO> 
     */
    List<IndexDescDTO> listByEnterpriseAndModuleDetail(@Param("enterpriseId") Integer enterpriseId, @Param("moduleId") String moduleId, @Param("indexGroupName") String indexGroupName, @Param("userId") Integer userId, @Param("indexStatus") Integer indexStatus);

//    List<IndexDescDTO> listRecycle(@Param("enterpriseId") Integer enterpriseId, @Param("moduleId") String moduleId, @Param("indexGroupName") String indexGroupName, @Param("userId") Integer userId);

    /**
     * 查询指标
     * @Title: listAll 
     * @Description:
     * @author zhiwj  
     * @return  java.util.List<com.gic.cloud.entity.TabIndexDesc> 
     */
    List<TabIndexDesc> listAll();

    /**
     * 查询指标
     * @Title: listAllUnRel 
     * @Description:
     * @author zhiwj
     * @param relIndexDescIdList 
     * @return  java.util.List<com.gic.cloud.entity.TabIndexDesc> 
     */
    List<TabIndexDesc> listAllUnRel(@Param("ids") List<Integer> relIndexDescIdList);

    /**
     * 查询指标
     * @Title: listByIds 
     * @Description:
     * @author zhiwj
     * @param indexDescIdList 
     * @return  java.util.List<com.gic.cloud.entity.TabIndexDesc> 
     */
    List<TabIndexDesc> listByIds(@Param("ids") List<Integer> indexDescIdList);

    /**
     * 查询指标
     * @Title: getIndexDescByIndexCode 
     * @Description:
     * @author zhiwj
     * @param moduleId
     * @param indexCode
     * @param indexGroupName 
     * @return  com.gic.cloud.entity.TabIndexDesc 
     */
    TabIndexDesc getIndexDescByIndexCode(@Param("moduleId") String moduleId, @Param("indexCode") String indexCode, @Param("indexGroupName") String indexGroupName);
}