package com.gic.cloud.dao.mapper;

import com.gic.cloud.entity.TabSysAccountGroup;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 账号分组
 * @ClassName: TabSysAccountGroupMapper 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 10:31 AM 
 */
public interface TabSysAccountGroupMapper {


    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysAccountGroup record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysAccountGroup record);

    /**
     * 根据主键查询
     *
     * @param accountGroupId 主键
     * @return 实体对象
     */
    TabSysAccountGroup selectByPrimaryKey(Integer accountGroupId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysAccountGroup record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysAccountGroup record);

    /**
     * 根据账号分组ID和账号分组名称查询数量，为了判断是否账号名称重复
     * @Title: countByAccountGroupName 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param accountGroupId
     * @param accountGroupName 
     * @return  int 
     */
    int countByAccountGroupName(@Param("enterpriseId") Integer enterpriseId,
                                @Param("accountGroupId") Integer accountGroupId,
                                @Param("accountGroupName") String accountGroupName);

    /**
     * 查询账号分组在商户下当前的最大排序值
     * @Title: queryMaxSort 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  int 
     */
    int queryMaxSort(@Param("enterpriseId") Integer enterpriseId);

    /**
     * 查询商户下所有的账号分组列表
     * @Title: listAccountGroupOrderBySort 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  java.util.List<com.gic.cloud.entity.TabSysAccountGroup> 
     */
    List<TabSysAccountGroup> listAccountGroupOrderBySort(@Param("enterpriseId") Integer enterpriseId);

    /**
     * 删除账号分组
     * @Title: deleteAccountGroup 
     * @Description:
     * @author guojuxing
     * @param accountGroupId 
     * @return  int 
     */
    int deleteAccountGroup(@Param("accountGroupId")Integer accountGroupId);
}