package com.gic.cloud.dao.mapper;

import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.entity.TabSysUser;
import com.gic.cloud.qo.UserQo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
/**
 * 用户
 * @ClassName: TabSysUserMapper 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 10:32 AM 
 */
public interface TabSysUserMapper {


    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysUser record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysUser record);

    /**
     * 根据主键查询
     *
     * @param userId 主键
     * @return 实体对象
     */
    TabSysUser selectByPrimaryKey(Integer userId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysUser record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysUser record);

    /**
     * 根据手机号查询用户个数，用于判断用户是否重复
     * @Title: countByRepeatPhone 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param userId
     * @param nationCode
     * @param phone 
     * @return  int 
     */
    int countByRepeatPhone(@Param("enterpriseId") Integer enterpriseId,
                           @Param("userId") Integer userId,
                           @Param("nationCode") String nationCode,
                           @Param("phone") String phone);

    /**
     * 删除用户
     * @Title: deleteUser 
     * @Description:
     * @author guojuxing
     * @param userId 
     * @return  void 
     */
    void deleteUser(@Param("userId") Integer userId);

    /**
     * 删除账号分组下的所有用户
     * @Title: deleteUserByAccountGroupId 
     * @Description:
     * @author guojuxing
     * @param accountGroupId 
     * @return  void 
     */
    void deleteUserByAccountGroupId(@Param("accountGroupId") Integer accountGroupId);


    /**
     * 分页查询用户列表
     * @Title: pageUser 
     * @Description:
     * @author guojuxing
     * @param userQo 
     * @return  java.util.List<com.gic.cloud.entity.TabSysUser> 
     */
    List<TabSysUser> pageUser(UserQo userQo);


    /**
     * 批量转移账号分组
     * @Title: bulkTransferAccountGroup 
     * @Description:
     * @author guojuxing
     * @param userIdList
     * @param targetAccountGroupId 
     * @return  void 
     */
    void bulkTransferAccountGroup(@Param("userIdList") List<Integer> userIdList, @Param("targetAccountGroupId") Integer targetAccountGroupId);


    /**
     * 查询账号下用户个数
     * @Title: countGroupByAccountGroupId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  java.util.List<com.gic.cloud.dto.UserDTO> 
     */
    List<UserDTO> countGroupByAccountGroupId(@Param("enterpriseId")Integer enterpriseId);

    /**
     * 手机号查询用户
     * @Title: listUserByPhone 
     * @Description:
     * @author guojuxing
     * @param nationCode
     * @param phone 
     * @return  java.util.List<com.gic.cloud.entity.TabSysUser> 
     */
    List<TabSysUser> listUserByPhone(@Param("nationCode") String nationCode, @Param("phone") String phone);


    /**
     * 查询用户列表（功能权限授权个数）
     * @Title: countGroupByFunctionAuthId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param functionIdList 
     * @return  java.util.List<com.gic.cloud.dto.UserDTO> 
     */
    List<UserDTO> countGroupByFunctionAuthId(@Param("enterpriseId") Integer enterpriseId,
                                             @Param("functionIdList") List<Integer> functionIdList);

    /**
     * 查询用户列表（功能权限授权个数）
     * @Title: countGroupByDataAuthId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param dataAuthIdList 
     * @return  java.util.List<com.gic.cloud.dto.UserDTO> 
     */
    List<UserDTO> countGroupByDataAuthId(@Param("enterpriseId") Integer enterpriseId,
                                         @Param("dataAuthIdList") List<Integer> dataAuthIdList);
}