package com.gic.cloud.service;

import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.entity.TabSysUser;
import com.gic.cloud.qo.UserQo;
import com.github.pagehelper.Page;

import java.util.List;
/**
 * 用户
 * @ClassName: UserService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 10:32 AM 
 */
public interface UserService {


    /**
     * 新增用户
     * @Title: saveUser 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  java.lang.Integer 
     */
    Integer saveUser(UserDTO dto);


    /**
     * 编辑用户信息
     * @Title: editUser 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  void 
     */
    void editUser(UserDTO dto);


    /**
     * 查询用户信息根据主键
     * @Title: getUser 
     * @Description:
     * @author guojuxing
     * @param userId 
     * @return  com.gic.cloud.entity.TabSysUser 
     */
    TabSysUser getUser(Integer userId);


    /**
     * 手机号是否重复
     * @Title: isRepeatPhone 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param userId
     * @param nationCode
     * @param phone 
     * @return  boolean 
     */
    boolean isRepeatPhone(Integer enterpriseId, Integer userId, String nationCode, String phone);


    /**
     * 删除用户根据主键
     * @Title: deleteUser 
     * @Description:
     * @author guojuxing
     * @param userId 
     * @return  void 
     */
    void deleteUser(Integer userId);

    /**
     * 删除账号分组下的所有用户
     * @Title: deleteUserByAccountGroupid 
     * @Description:
     * @author guojuxing
     * @param accountGroupId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    void deleteUserByAccountGroupId(Integer accountGroupId);


    /**
     * 分页查询用户列表
     * @Title: pageUser 
     * @Description:
     * @author guojuxing
     * @param userQo 
     * @return  com.github.pagehelper.Page<com.gic.cloud.entity.TabSysUser> 
     */
    Page<TabSysUser> pageUser(UserQo userQo);


    /**
     * 查询用户列表
     * @Title: listUser 
     * @Description:
     * @author guojuxing
     * @param userQo 
     * @return  java.util.List<com.gic.cloud.entity.TabSysUser> 
     */
    List<TabSysUser> listUser(UserQo userQo);


    /**
     * 批量转移账号分组
     * @Title: bulkTransferAccountGroup 
     * @Description:
     * @author guojuxing
     * @param userIdList
     * @param targetAccountGroupId 
     * @return  void 
     */
    void bulkTransferAccountGroup(List<Integer> userIdList, Integer targetAccountGroupId);


    /**
     * 账号列表（每个账号下用户个数）
     * @Title: countGroupByAccountGroupId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId 
     * @return  java.util.List<com.gic.cloud.dto.UserDTO> 
     */
    List<UserDTO> countGroupByAccountGroupId(Integer enterpriseId);


    /**
     * 手机号查询用户
     * @Title: listUserByPhone 
     * @Description:
     * @author guojuxing
     * @param nationCode
     * @param phone 
     * @return  java.util.List<com.gic.cloud.entity.TabSysUser> 
     */
    List<TabSysUser> listUserByPhone(String nationCode, String phone);


    /**
     * 用户列表（功能权限授权个数）
     * @Title: countGroupByFunctionAuthId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param functionIdList 
     * @return  java.util.List<com.gic.cloud.dto.UserDTO> 
     */
    List<UserDTO> countGroupByFunctionAuthId(Integer enterpriseId, List<Integer> functionIdList);


    /**
     * 用户列（数据权限授权个数）
     * @Title: countGroupByDataAuthId 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param dataAuthIdList 
     * @return  java.util.List<com.gic.cloud.dto.UserDTO> 
     */
    List<UserDTO> countGroupByDataAuthId(Integer enterpriseId, List<Integer> dataAuthIdList);
}
