package com.gic.cloud.service.impl;

import com.gic.cloud.constanst.DeleteFlagConstants;
import com.gic.cloud.dao.mapper.TabIndexDescMapper;
import com.gic.cloud.dto.IndexDescDTO;
import com.gic.cloud.entity.TabIndexDesc;
import com.gic.cloud.entity.TabIndexDescEnterprise;
import com.gic.cloud.entity.TabUpdateTips;
import com.gic.cloud.service.IndexDescEnterpriseService;
import com.gic.cloud.service.IndexDescService;
import com.gic.cloud.service.UpdateTipService;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-23 15:11
 */
@Service
public class IndexDescServiceImpl implements IndexDescService {

    @Autowired
    private TabIndexDescMapper tabIndexDescMapper;
    @Autowired
    private IndexDescEnterpriseService indexDescEnterpriseService;
    @Autowired
    private UpdateTipService updateTipService;

    @Override
    public List<IndexDescDTO> listByEnterpriseAndModule(Integer enterpriseId, String moduleId, String indexGroupName) {
        return tabIndexDescMapper.listByEnterpriseAndModule(enterpriseId, moduleId, indexGroupName);
    }

    @Override
    public List<IndexDescDTO> listByEnterpriseAndModuleDetail(Integer enterpriseId, String moduleId, String indexGroupName, Integer userId) {
        List<IndexDescDTO> indexDescDTOList = tabIndexDescMapper.listByEnterpriseAndModuleDetail(enterpriseId, moduleId, indexGroupName, userId, 1);
        return parse(indexDescDTOList, userId);
    }

    @Override
    public List<IndexDescDTO> listRecycle(Integer enterpriseId, String moduleId, String indexGroupName, Integer userId) {
        List<IndexDescDTO> indexDescDTOList = tabIndexDescMapper.listByEnterpriseAndModuleDetail(enterpriseId, moduleId, indexGroupName, userId, 0);
        return parse(indexDescDTOList, userId);
    }

    private List<IndexDescDTO> parse(List<IndexDescDTO> indexDescDTOList, Integer userId) {
        if (CollectionUtils.isNotEmpty(indexDescDTOList)) {
            List<Integer> indexIdList = indexDescDTOList.stream().map(IndexDescDTO::getIndexId).collect(Collectors.toList());
            Map<Integer, TabUpdateTips> updateTipsMap = updateTipService.mapByIndexIdAndUser(indexIdList, 1, userId);
            for (IndexDescDTO indexDescDTO : indexDescDTOList) {
//                --     ,ifnull(t4.show_status, 2) updateTips
//                        --     ,t4.update_tips_id updateTipsId
                TabUpdateTips updateTips = updateTipsMap.get(indexDescDTO.getIndexId());
                if (updateTips != null) {
                    indexDescDTO.setUpdateTips(updateTips.getShowStatus());
                    indexDescDTO.setUpdateTipsId(updateTips.getUpdateTipsId());
                } else {
                    indexDescDTO.setUpdateTips(2);
                    indexDescDTO.setUpdateTipsId(null);
                }
            }
        }
        return indexDescDTOList;
    }

    @Override
    public void sort(Integer enterpriseId, String moduleId, String indexGroupName, Integer indexDescEnterpriseId, Integer seq) {
        PageHelper.offsetPage(seq - 1, 1);
        List<IndexDescDTO> dtoList = tabIndexDescMapper.listByEnterpriseAndModuleDetail(enterpriseId, moduleId, indexGroupName, null, 1);
        IndexDescDTO indexDescDTO = dtoList.get(0);
        indexDescEnterpriseService.updateSort(enterpriseId, indexDescEnterpriseId, indexDescDTO.getSort());
    }

    @Override
    public TabIndexDesc getIndexDesc(Integer enterpriseId, Integer indexDescId) {
        return tabIndexDescMapper.selectByPrimaryKey(indexDescId);
    }

    @Override
    public List<TabIndexDesc> listAllUnRel(List<Integer> indexDescIdList) {
        return tabIndexDescMapper.listAllUnRel(indexDescIdList);
    }

    @Override
    public List<TabIndexDesc> listByIds(List<Integer> indexDescIdList) {
        return tabIndexDescMapper.listByIds(indexDescIdList);
    }

    @Override
    public TabIndexDesc getIndexDesc(String moduleId, String indexCode, String indexGroupName) {
        return tabIndexDescMapper.getIndexDescByIndexCode(moduleId, indexCode, indexGroupName);
    }

    @Override
    public void init(Integer enterpriseId) {
        indexDescEnterpriseService.delete(enterpriseId);
        List<TabIndexDesc> indexDescDTOList = tabIndexDescMapper.listAll();
        for (TabIndexDesc indexDescDTO : indexDescDTOList) {
            TabIndexDescEnterprise indexDescEnterprise = new TabIndexDescEnterprise();
            indexDescEnterprise.setIndexDescId(indexDescDTO.getIndexDescId());
            indexDescEnterprise.setEnterpriseId(enterpriseId);
            indexDescEnterprise.setIndexStatus(1);
            indexDescEnterprise.setSort(0);
            indexDescEnterprise.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
            indexDescEnterpriseService.insert(indexDescEnterprise);
        }
    }
}
