package com.gic.cloud.service.impl;

import com.gic.cloud.constants.LogAndUpdateTipsTypeEnum;
import com.gic.cloud.dao.mapper.TabUpdateTipsMapper;
import com.gic.cloud.entity.TabDataExplain;
import com.gic.cloud.entity.TabIndex;
import com.gic.cloud.entity.TabSysUser;
import com.gic.cloud.entity.TabUpdateTips;
import com.gic.cloud.qo.DataExplainQo;
import com.gic.cloud.qo.IndexQo;
import com.gic.cloud.qo.UserQo;
import com.gic.cloud.service.DataExplainService;
import com.gic.cloud.service.IndexService;
import com.gic.cloud.service.UpdateTipService;
import com.gic.cloud.service.UserService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-08 14:28
 */
@Service
public class UpdateTipServiceImpl implements UpdateTipService {

    @Autowired
    private TabUpdateTipsMapper tabUpdateTipsMapper;
    @Autowired
    private UserService userService;
    @Autowired
    private IndexService indexService;
    @Autowired
    private DataExplainService dataExplainService;

    @Override
    public void insertBySaveUser(Integer userId) {
        List<TabIndex> indexList = indexService.listAllIndex(new IndexQo());
        for (TabIndex index : indexList) {
            TabUpdateTips tabUpdateTips = new TabUpdateTips();
            tabUpdateTips.setUserId(userId);
            tabUpdateTips.setBusinessId(index.getIndexId());
            tabUpdateTips.setShowStatus(1);
            tabUpdateTips.setType(LogAndUpdateTipsTypeEnum.INDEX.getCode());
            tabUpdateTipsMapper.insertSelective(tabUpdateTips);
        }
        List<TabDataExplain> dataExplainList = dataExplainService.listAllDataExplain(new DataExplainQo());
        for (TabDataExplain dataExplain : dataExplainList) {
            TabUpdateTips tabUpdateTips = new TabUpdateTips();
            tabUpdateTips.setUserId(userId);
            tabUpdateTips.setBusinessId(dataExplain.getDataExplainId());
            tabUpdateTips.setShowStatus(1);
            tabUpdateTips.setType(LogAndUpdateTipsTypeEnum.DATA_EXPLAIN.getCode());
            tabUpdateTipsMapper.insertSelective(tabUpdateTips);
        }
    }

    @Override
    public void insertBySaveBusiness(Integer businessId, Integer type) {
        List<TabSysUser> users = userService.listUser(new UserQo());
        for (TabSysUser user : users) {
            TabUpdateTips tabUpdateTips = new TabUpdateTips();
            tabUpdateTips.setUserId(user.getUserId());
            tabUpdateTips.setBusinessId(businessId);
            tabUpdateTips.setShowStatus(1);
            tabUpdateTips.setType(type);
            tabUpdateTipsMapper.insertSelective(tabUpdateTips);
        }
    }

    @Override
    public void publish(Integer businessId, Integer type) {
        tabUpdateTipsMapper.updateStatus(businessId, type, null, 1);
    }

    @Override
    public void erase(Integer userId, Integer updateTipsId, Integer type) {
        TabUpdateTips tabUpdateTips = new TabUpdateTips();
        tabUpdateTips.setUpdateTipsId(updateTipsId);
        tabUpdateTips.setShowStatus(0);
        tabUpdateTipsMapper.updateByPrimaryKeySelective(tabUpdateTips);
    }

    @Override
    public Map<Integer, TabUpdateTips> mapByIndexIdAndUser(List<Integer> indexIdList, int type, Integer userId) {
        if (CollectionUtils.isNotEmpty(indexIdList)) {
            List<TabUpdateTips> updateTipsList = tabUpdateTipsMapper.listByIndexIdAndUser(indexIdList, type, userId);
            if (CollectionUtils.isNotEmpty(updateTipsList)) {
                return updateTipsList.stream().collect(Collectors.toMap(TabUpdateTips::getBusinessId, e -> e, (a, b) -> a));
            }
        }
        return Collections.emptyMap();

    }

}
