package com.gic.cloud.service.impl;

import com.gic.cloud.constanst.DeleteFlagConstants;
import com.gic.cloud.dao.mapper.TabSysUserMapper;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.entity.TabSysUser;
import com.gic.cloud.qo.UserQo;
import com.gic.cloud.service.UserService;
import com.gic.commons.util.EntityUtil;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
/**
 * 用户
 * @ClassName: UserServiceImpl 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 10:32 AM 
 */
@Service("userService")
public class UserServiceImpl implements UserService{
    @Autowired
    private TabSysUserMapper tabSysUserMapper;

    @Override
    public Integer saveUser(UserDTO dto) {
        TabSysUser record = EntityUtil.changeEntityNew(TabSysUser.class, dto);
        Date now = new Date();
        record.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
        record.setCreateTime(now);
        record.setUpdateTime(now);
        tabSysUserMapper.insert(record);
        return record.getUserId();
    }

    @Override
    public void editUser(UserDTO dto) {
        TabSysUser record = EntityUtil.changeEntityNew(TabSysUser.class, dto);
        record.setUpdateTime(new Date());
        tabSysUserMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public TabSysUser getUser(Integer userId) {
        return tabSysUserMapper.selectByPrimaryKey(userId);
    }

    @Override
    public boolean isRepeatPhone(Integer enterpriseId, Integer userId, String nationCode, String phone) {
        if (tabSysUserMapper.countByRepeatPhone(enterpriseId, userId, nationCode, phone) > 0) {
            return true;
        }
        return false;
    }

    @Override
    public void deleteUser(Integer userId) {
        tabSysUserMapper.deleteUser(userId);
    }

    @Override
    public void deleteUserByAccountGroupId(Integer accountGroupId) {
        tabSysUserMapper.deleteUserByAccountGroupId(accountGroupId);
    }

    @Override
    public Page<TabSysUser> pageUser(UserQo userQo) {
        PageHelper.startPage(userQo.getCurrentPage(), userQo.getPageSize());
        return (Page<TabSysUser>) tabSysUserMapper.pageUser(userQo);
    }

    @Override
    public List<TabSysUser> listUser(UserQo userQo) {
        return tabSysUserMapper.pageUser(userQo);
    }

    @Override
    public void bulkTransferAccountGroup(List<Integer> userIdList, Integer targetAccountGroupId) {
        if (CollectionUtils.isEmpty(userIdList)) {
            return ;
        }
        tabSysUserMapper.bulkTransferAccountGroup(userIdList, targetAccountGroupId);
    }

    @Override
    public List<UserDTO> countGroupByAccountGroupId(Integer enterpriseId) {
        return tabSysUserMapper.countGroupByAccountGroupId(enterpriseId);
    }

    @Override
    public List<TabSysUser> listUserByPhone(String nationCode, String phone) {
        return tabSysUserMapper.listUserByPhone(nationCode, phone);
    }

    @Override
    public List<UserDTO> countGroupByFunctionAuthId(Integer enterpriseId, List<Integer> functionIdList) {
        return tabSysUserMapper.countGroupByFunctionAuthId(enterpriseId, functionIdList);
    }

    @Override
    public List<UserDTO> countGroupByDataAuthId(Integer enterpriseId, List<Integer> dataAuthIdList) {
        return tabSysUserMapper.countGroupByDataAuthId(enterpriseId, dataAuthIdList);
    }
}
