package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.AccountGroupDTO;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.entity.TabSysAccountGroup;
import com.gic.cloud.service.AccountGroupApiService;
import com.gic.cloud.service.AccountGroupService;
import com.gic.cloud.service.UserService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Title:  
 * @Description:
 * @author guojuxing
 * @return   
 */
@Service("accountGroupApiService")
public class AccountGroupApiServiceImpl implements AccountGroupApiService{
    @Autowired
    private AccountGroupService accountGroupService;
    @Autowired
    private UserService userService;

    @Override
    public ServiceResponse<Integer> saveAccountGroup(AccountGroupDTO dto) {
        //valid param
        String accountGroupName = dto.getAccountGroupName();
        Integer enterpriseId = dto.getEnterpriseId();
        if (StringUtils.isBlank(accountGroupName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组名称不能为空");
        }
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (accountGroupService.isRepeatAccountGroupName(enterpriseId, null, accountGroupName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组名称重复");
        }
        dto.setSort(accountGroupService.queryMaxSort(enterpriseId) + 1);
        return ServiceResponse.success(accountGroupService.saveAccountGroup(dto));
    }

    @Override
    public ServiceResponse<Void> editAccountGroup(AccountGroupDTO dto) {
        Integer accountGroupId = dto.getAccountGroupId();
        TabSysAccountGroup record = accountGroupService.getAccountGroupById(accountGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组ID错误，查无数据");
        }
        String accountGroupName = dto.getAccountGroupName();
        if (StringUtils.isBlank(accountGroupName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组名称不能为空");
        }
        if (accountGroupService.isRepeatAccountGroupName(record.getEnterpriseId(), accountGroupId, accountGroupName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组名称重复");
        }
        dto.setEnterpriseId(record.getEnterpriseId());
        dto.setSort(record.getSort());
        accountGroupService.editAccountGroup(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> sortAccountGroup(Integer accountGroupId, Integer targetSort) {
        TabSysAccountGroup record = accountGroupService.getAccountGroupById(accountGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组ID错误，查无数据");
        }
        int maxSort = accountGroupService.queryMaxSort(record.getEnterpriseId());
        if (targetSort > maxSort || targetSort < 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "排序值非法");
        }
        accountGroupService.sortAccountGroup(record, targetSort);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<AccountGroupDTO>> listAccountGroupOfCountMember(Integer enterpriseId) {
        List<TabSysAccountGroup> list = accountGroupService.listAccountGroupOrderBySort(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            List<AccountGroupDTO> result = EntityUtil.changeEntityListNew(AccountGroupDTO.class, list);
            //统计用户数量
            List<UserDTO> userList = userService.countGroupByAccountGroupId(enterpriseId);
            if (CollectionUtils.isNotEmpty(userList)) {
                Map<String, Integer> map = userList.stream()
                        .collect(Collectors.toMap(e -> e.getAccountGroupId().toString(), e -> e.getMemberCount()));
                result.forEach(e -> e.setMemberCount(map.get(e.getAccountGroupId().toString())));
            }
            return ServiceResponse.success(result);
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<Void> deleteAccountGroup(Integer accountGroupId) {
        TabSysAccountGroup record = accountGroupService.getAccountGroupById(accountGroupId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组ID错误，查无数据");
        }
        accountGroupService.deleteAccountGroup(accountGroupId);
        //删除分组下的用户
        userService.deleteUserByAccountGroupId(accountGroupId);
        return ServiceResponse.success();
    }
}
