package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.constanst.DeleteFlagConstants;
import com.gic.cloud.dao.mapper.EnterprisePortraitRelMapper;
import com.gic.cloud.dto.EnterprisePortraitRelDTO;
import com.gic.cloud.entity.TabGicEnterprisePortraitRel;
import com.gic.cloud.service.EnterprisePortraitRelApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;
/**
 * 会员画像
 * @ClassName: EnterprisePortraitRelApiServiceImpl 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 10:45 AM 
 */
@Service("enterprisePortraitRelApiService")
public class EnterprisePortraitRelApiServiceImpl implements EnterprisePortraitRelApiService{
    @Autowired
    private EnterprisePortraitRelMapper enterprisePortraitRelMapper;

    @Override
    public ServiceResponse<List<EnterprisePortraitRelDTO>> listPortraitField(Integer enterpriseId, Integer dataType, Integer status) {
        return ServiceResponse.success(Optional.ofNullable(EntityUtil.changeEntityListNew(EnterprisePortraitRelDTO.class,
                enterprisePortraitRelMapper.listPortraitField(enterpriseId, dataType, status))).orElse(Collections.emptyList()));
    }

    @Override
    public ServiceResponse<Void> insertForeach(List<EnterprisePortraitRelDTO> list) {
        if (CollectionUtils.isNotEmpty(list)) {
            Integer enterpriseId = list.get(0).getEnterpriseId();
            List<TabGicEnterprisePortraitRel> hasList = enterprisePortraitRelMapper.listPortraitField(enterpriseId, 1, null);
            if (CollectionUtils.isNotEmpty(hasList)) {
                return ServiceResponse.success();
            }
        }
        enterprisePortraitRelMapper.insertForeach(list);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> savePortraitField(Integer enterpriseId, String fieldCode, String fieldName, Integer dataType) {
        //查询是否已经存在
        TabGicEnterprisePortraitRel record = enterprisePortraitRelMapper.getPortraitField(enterpriseId, fieldCode, dataType);
        if (record == null) {
            //不存在
            int maxSort = enterprisePortraitRelMapper.getMaxSort(enterpriseId, dataType);
            enterprisePortraitRelMapper.insert(new TabGicEnterprisePortraitRel()
                    .setCreateTime(new Date())
                    .setDataType(dataType)
                    .setEnterpriseId(enterpriseId)
                    .setPortraitCode(fieldCode)
                    .setPortraitName(fieldName)
                    .setStatus(1)
                    .setUpdateTime(new Date())
                    .setSort(maxSort + 1)
                    .setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS));
        } else {
            if (!record.getPortraitName().equals(fieldName)) {
                //如果名称有变动，则更新
                enterprisePortraitRelMapper.updateByPrimaryKeySelective(new TabGicEnterprisePortraitRel()
                        .setPortraitRelationId(record.getPortraitRelationId())
                        .setPortraitName(fieldName));
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> sort(Integer enterpriseId, String fieldCode, Integer dataType, Integer sort) {
        TabGicEnterprisePortraitRel record = enterprisePortraitRelMapper.getPortraitField(enterpriseId, fieldCode, dataType);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数错误，查无数据");
        }
        List<TabGicEnterprisePortraitRel> list = enterprisePortraitRelMapper.listByDataType(enterpriseId, dataType);
        Integer fromSortValue = record.getSort();
        if (fromSortValue > sort) {
            //向上拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabGicEnterprisePortraitRel temp = list.get(i);
                //如果大于sortValue，都需要降低排序值,往后推
                boolean isNeedDown = temp.getSort() >= sort && temp.getSort() < fromSortValue;
                if (isNeedDown) {
                    updateSort(list.get(i + 1).getSort(), temp.getPortraitRelationId());
                }
            }
        } else if (fromSortValue < sort) {
            //向下拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabGicEnterprisePortraitRel temp = list.get(i);
                boolean isNeedUp = temp.getSort() <= sort && temp.getSort() > fromSortValue;
                if (isNeedUp) {
                    updateSort(list.get(i - 1).getSort(), temp.getPortraitRelationId());
                }
            }
        }
        updateSort(sort, record.getPortraitRelationId());
        return null;
    }

    @Override
    public ServiceResponse<Void> updateStatus(Integer enterpriseId, String fieldCode, Integer dataType, Integer status) {
        TabGicEnterprisePortraitRel record = enterprisePortraitRelMapper.getPortraitField(enterpriseId, fieldCode, dataType);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数错误，查无数据");
        }
        if (status == 0) {
            //放到回收站
            enterprisePortraitRelMapper.updateByPrimaryKeySelective(new TabGicEnterprisePortraitRel()
                    .setPortraitRelationId(record.getPortraitRelationId())
                    .setStatus(2));

            List<TabGicEnterprisePortraitRel> list = enterprisePortraitRelMapper.listByDataType(enterpriseId, dataType);
            Integer fromSortValue = record.getSort();
            for (TabGicEnterprisePortraitRel temp : list) {
                if (temp.getSort() > fromSortValue) {
                    //排在下面的都上升一个序号
                    updateSort(temp.getSort() + 1, temp.getPortraitRelationId());
                }
            }
        } else {
            int maxSort = enterprisePortraitRelMapper.getMaxSort(enterpriseId, dataType);
            //复原，排序放在最后一位
            enterprisePortraitRelMapper.updateByPrimaryKeySelective(new TabGicEnterprisePortraitRel()
                    .setPortraitRelationId(record.getPortraitRelationId())
                    .setStatus(1)
                    .setSort(maxSort + 1));
        }
        return null;
    }

    @Override
    public ServiceResponse<Void> delete(Integer portraitRelationId) {
        enterprisePortraitRelMapper.updateByPrimaryKeySelective(new TabGicEnterprisePortraitRel()
                .setPortraitRelationId(portraitRelationId)
                .setStatus(0)
                .setDeleteFlag(DeleteFlagConstants.DELETE_STATUS));
        return ServiceResponse.success();
    }

    private void updateSort(Integer sort, Integer id) {
        enterprisePortraitRelMapper.updateByPrimaryKeySelective(new TabGicEnterprisePortraitRel()
                .setPortraitRelationId(id)
                .setSort(sort));
    }

}
