package com.gic.cloud.web.controller;

import com.gic.store.service.StoreDictApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.TempStoreConditionDTO;
import com.gic.cloud.service.TempStoreConditionApiService;
import com.gic.cloud.web.vo.TempStoreConditionDetailVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreWidgetApiService;
/**
 * 门店暂存区
 * @ClassName: TempStoreConditionController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 10:59 AM 
 */
@RestController
@RequestMapping("temp-store")
public class TempStoreConditionController {
    private static Logger LOGGER = LogManager.getLogger(TempStoreConditionController.class);

    @Autowired
    private TempStoreConditionApiService tempStoreConditionApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreDictApiService storeDictApiService;

    @RequestMapping("/save-or-update-temp-store-condition")
    public RestResponse saveTempStoreCondition(TempStoreConditionDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(tempStoreConditionApiService.saveOrUpdateTempStoreCondition(dto));
    }

    @RequestMapping("/get-temp-store-condition")
    public RestResponse getTempStoreCondition() {
        return ResultControllerUtils.commonResultOne(tempStoreConditionApiService
                .getTempStoreCondition(UserDetailUtils.getUserDetail().getEnterpriseId()), TempStoreConditionDetailVO.class);
    }

    @RequestMapping("/list-store-status")
    public RestResponse listStoreStatus() {
        return ResultControllerUtils.commonResult(storeDictApiService
                .listAllStoreStatus(UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    @RequestMapping("/query-temp-store")
    public RestResponse queryTempStore(@RequestBody StoreSearchDTO storeSearchDTO) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        ServiceResponse<TempStoreConditionDTO> result = tempStoreConditionApiService.getTempStoreCondition(enterpriseId);
        if (result.isSuccess()) {
            TempStoreConditionDTO tempStoreConditionDTO = result.getResult();
            if (tempStoreConditionDTO != null) {
                StoreWidgetDTO storeWidgetDTO = this.storeWidgetApiService.getStoreWidget(tempStoreConditionDTO.getStoreWidgetId()).getResult();
                if (storeWidgetDTO != null) {
                    String authSearchParam = storeWidgetDTO.getAuthSearchParam();
                    String searchParam = storeWidgetDTO.getSearchParam();
                    String authSearchParamSign = "{}";
                    if (StringUtils.isNotBlank(authSearchParam) && !authSearchParamSign.equals(authSearchParam)) {
                        storeSearchDTO.setAuthSearchJson(authSearchParam);
                    }
                    storeSearchDTO.setSearchJson(searchParam);
                }
            }
        }
        return ResultControllerUtils.commonResult(storeApiService.listStore(storeSearchDTO, storeSearchDTO.getCurrentPage(), storeSearchDTO.getPageSize()));
    }
}
