package com.gic.cloud.web.auth;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.*;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.UserResourceApiService;
import com.gic.cloud.dto.DataAuthDTO;
import com.gic.cloud.service.DataAuthApiService;
import com.gic.cloud.web.constant.StoreChannelEnum;
import com.gic.cloud.web.vo.MbrVo;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.service.UnionEnterpriseAuthApiService;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.member.config.api.dto.AreaSearchDTO;
import com.gic.member.config.api.dto.ChannelAreaInfoDTO;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.UserListFieldApiService;
import com.gic.platform.member.constant.enums.CuTypeEnum;
import com.gic.store.dto.StoreDTO;
import com.gic.store.dto.StoreSearchDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
/**
 * 数据权限工具
 * @ClassName: DataAuthUtils 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 11:43 AM 
 */
@Component
public class DataAuthUtils {
    private static final Logger log = LogManager.getLogger(DataAuthUtils.class);
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private UnionEnterpriseAuthApiService unionEnterpriseAuthApiService;
    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private UserListFieldApiService userListFieldApiService;

    public StoreAuth getStoreAuth(Integer userId, Integer enterpriseId){
        StoreAuth storeAuth = new StoreAuth();
        StoreResourceDTO storeResourceDTO = resourceGroupApiService.getStoreResourceByUserId(userId).getResult();
        if(storeResourceDTO.getType() == 1){
            //所有门店权限
            storeAuth.setHasAuth(true);
            List<StoreDTO> storeList = storeApiService.listStore(null, enterpriseId).getResult();
            if(CollectionUtils.isNotEmpty(storeList)){
                List<Integer> storeInfoIdList = storeList.stream().map(storeDTO -> storeDTO.getStoreInfoId()).collect(Collectors.toList());
                storeAuth.setStoreInfoIdList(storeInfoIdList);
            }
            return storeAuth;
        }
        if(CollectionUtils.isEmpty(storeResourceDTO.getStoreIdList())){
            //没有门店权限
            storeAuth.setHasAuth(false);
            storeResourceDTO.getStoreIdList().add(-99);
            storeAuth.setStoreInfoIdList(storeResourceDTO.getStoreIdList());
            return storeAuth;
        }
        storeAuth.setHasAuth(true);
        storeAuth.setStoreInfoIdList(storeResourceDTO.getStoreIdList());
        storeAuth.setStoreWidgetId(storeResourceDTO.getStoreResourceId());
        return storeAuth;
    }


    public OnLineAuth getOnlineStore(Integer userId, Integer enterpriseId){
        ResourceGroupDataDTO resourceGroupDataDTO = resourceGroupApiService.getResourceGroupByUserId(userId).getResult();
        CustomUserAreaDTO customUserAreaDTO = resourceGroupDataDTO.getCustomUserAreaDTO();
        OnLineAuth onLineAuth = new OnLineAuth();
        onLineAuth.setHasAuth(true);
        OnLineStore onLineStore = new OnLineStore();
        onLineStore.setChannel(StoreChannelEnum.WMMALL.getChannel());
        List<OnLineStore> list = new ArrayList<>();
        if(customUserAreaDTO.getType() == 1){
            //超管需要展示 所有联合的线上店铺  当前商户所有线上店铺
            List<String> storeIdList = new ArrayList<>();
            List<UnionEnterpriseAuthDTO> unionEnterpriseAuthDTOS = unionEnterpriseAuthApiService.listOwnEnterpriseAuth(enterpriseId).getResult();
            if(CollectionUtils.isNotEmpty(unionEnterpriseAuthDTOS)){
                for(UnionEnterpriseAuthDTO dto : unionEnterpriseAuthDTOS){
                    Integer resourceGroupId = dto.getResourceGroupId();
                    ResourceGroupDTO resourceGroupDTO = resourceGroupApiService.getResourceGroup(resourceGroupId).getResult();
                    //线上店铺ids
                    String weimobStoreIds = resourceGroupDTO.getWeimobStoreId();
                    if(StringUtils.isNotBlank(weimobStoreIds)){
                        storeIdList.addAll(Arrays.asList(weimobStoreIds.split(",")));
                    }
                }
            }
            //查询当前商户下所有门店
            List<WmStoreDTO> result = wmStoreApiService.listWmStore(enterpriseId).getResult();
            if(CollectionUtils.isNotEmpty(result)){
                result.stream().forEach(wmStoreDTO -> storeIdList.add(wmStoreDTO.getWmMallStoreId()+""));
            }
            onLineStore.setStoreIdList(storeIdList);
            list.add(onLineStore);
            onLineAuth.setList(list);
            return onLineAuth;
        }
        List<Long> weimobStoreIdList = customUserAreaDTO.getWeimobStoreId();
        if(CollectionUtils.isEmpty(weimobStoreIdList)){
            if (weimobStoreIdList == null) {
                weimobStoreIdList = new ArrayList<>(1);
            }
            weimobStoreIdList.add(-99L);
            onLineAuth.setHasAuth(false);
            return onLineAuth;
        }
        onLineStore.setStoreIdList(weimobStoreIdList.stream().map(s -> String.valueOf(s)).collect(Collectors.toList()));
        list.add(onLineStore);
        onLineAuth.setList(list);
        return onLineAuth;
    }


    public List<MbrVo> getMbrArea(Integer userId, Integer enterpriseId){
        AreaSearchDTO areaSearchDTO = new AreaSearchDTO();
        areaSearchDTO.setCuTypes(Arrays.asList(CuTypeEnum.MCU));
        areaSearchDTO.setEnterpriseId(enterpriseId);
        areaSearchDTO.setUserId(userId);
        List<ChannelAreaInfoDTO> listCard = userListFieldApiService.queryCustomAreaInfo(areaSearchDTO).getResult();
        if(CollectionUtils.isNotEmpty(listCard)){
            return listCard.stream().map(dto -> {
                MbrVo vo = new MbrVo();
                vo.setMbrAreaId(dto.getAreaId().toString());
                vo.setMbrAreaName(dto.getName());
                vo.setOwnType(dto.getOwnerFlag());
                return vo;
            }).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    public List<Long> getMbrAreaId(Integer userId, Integer enterpriseId){
        List<MbrVo> mbrArea = getMbrArea(userId, enterpriseId);
        if(CollectionUtils.isNotEmpty(mbrArea)) {
            return mbrArea.stream().map(v -> Long.valueOf(v.getMbrAreaId())).collect(Collectors.toList());
        }
        ArrayList<Long> list = new ArrayList<>();
        list.add(-9999L);
        return list;
    }

}
