package com.gic.cloud.web.controller;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.web.auth.DataAuthUtils;
import com.gic.cloud.web.vo.BaseSalesVo;
import com.gic.cloud.web.vo.LevelVo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.data.api.dto.consumestructure.*;
import com.gic.data.api.qo.consumestructure.*;
import com.gic.data.api.service.consumestructure.ConsumeStructureApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
/**
 * 业绩构成
 * @ClassName: ConsumeStructureController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 1:36 PM 
 */
@RestController
public class ConsumeStructureController {
    private static final Logger log = LogManager.getLogger(ConsumeStructureController.class);
    @Autowired
    private ConsumeStructureApiService consumeStructureApiService;
    @Autowired
    private StoreSearchUtils storeSearchUtils;
    @Autowired
    private DataAuthUtils dataAuthUtils;
    @Autowired
    private GoDownComponent goDownComponent;

    /**
     * 年龄消费占比（订单人数/订单数/销售额）
     * @Title: queryAgeConsumeShare 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("get-age-consume-share")
    public RestResponse queryAgeConsumeShare(AgeConsumeShareQoAbstract params) {
        log.info("消费构成-消费年龄-年龄消费占比参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryAgeConsumeShare(params));
    }

    @RequestMapping("get-age-consume-survey")
    public RestResponse queryAgeConsumeSurvey(AgeConsumeShareQoAbstract params) {
        log.info("消费构成-消费年龄-概况参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryAgeConsumeSurvey(params));
    }

    @RequestMapping("get-age-consume-group")
    public RestResponse listAgeConsumeGroup(AgeConsumeAreaQoAbstract params) {
        log.info("消费构成-消费年龄-年龄段参数：{}", JSON.toJSONString(params));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        return ResultControllerUtils.commonResult(consumeStructureApiService.listAgeConsumeGroup(params));
    }

    @RequestMapping("get-age-consume-area")
    public RestResponse queryAgeConsumeArea(AgeConsumeAreaQoAbstract params) {
        log.info("消费构成-消费年龄-区域参数：{}", JSON.toJSONString(params));
        BaseSalesVo vo = new BaseSalesVo();
        LevelVo levelVo = this.goDownComponent.setValue(params, vo);
        Integer maxLevel = levelVo.getMaxLevel();
        params.setCountType(1);
        ServiceResponse<Page<AgeConsumeAreaDTO>> pageServiceResponse = consumeStructureApiService.pageAgeConsumeArea(params);
        if (!pageServiceResponse.isSuccess()) {
            return RestResponse.failure(pageServiceResponse.getCode(), pageServiceResponse.getMessage());
        }
        vo.setPage(this.goDownComponent.parseData(pageServiceResponse.getResult(), params.getChannel(), params.getEnterpriseId(), params.getLevel(), maxLevel));
        params.setCountType(2);
        goDownComponent.setValueTotalForArea(params, levelVo);
        ServiceResponse<Page<AgeConsumeAreaDTO>> ageConsumeAreaResponse = consumeStructureApiService.pageAgeConsumeArea(params);
        this.goDownComponent.parseTotal(ageConsumeAreaResponse, vo);
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel().intValue() == levelVo.getLevel()){
            //无归属
            params.setCountType(3);
            goDownComponent.setValueWgsForArea(params, levelVo);
            ServiceResponse<Page<AgeConsumeAreaDTO>> wgsResponse = consumeStructureApiService.pageAgeConsumeArea(params);
            this.goDownComponent.parseWgs(wgsResponse, vo);
        }
        return RestResponse.success(vo);
    }

    @RequestMapping("get-grade-consume-trend")
    public RestResponse getGradeConsumeTrend(GradeConsumeTrendQoAbstract params) {
        log.info("消费构成-等级-趋势参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryGradeConsumeTrend(params));
    }

    @RequestMapping("get-grade-consume-survey")
    public RestResponse getGradeConsumeSurvey(GradeConsumeTrendQoAbstract params) {
        log.info("消费构成-等级-概况参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryGradeConsumeSurvey(params));
    }

    @RequestMapping("get-grade-consume-area")
    public RestResponse queryGradeConsumeArea(GradeConsumeAreaQoAbstract params) {
        log.info("消费构成-等级-区域参数：{}", JSON.toJSONString(params));
        BaseSalesVo vo = new BaseSalesVo();
        LevelVo levelVo = this.goDownComponent.setValue(params, vo);
        Integer maxLevel = levelVo.getMaxLevel();
        params.setCountType(1);
        ServiceResponse<Page<GradeConsumeAreaDTO>> pageServiceResponse = consumeStructureApiService.pageGradeConsumeArea(params);
        if (!pageServiceResponse.isSuccess()) {
            return RestResponse.failure(pageServiceResponse.getCode(), pageServiceResponse.getMessage());
        }
        vo.setPage(this.goDownComponent.parseData(pageServiceResponse.getResult(), params.getChannel(), params.getEnterpriseId(), params.getLevel(), maxLevel));
        params.setCountType(2);
        goDownComponent.setValueTotalForArea(params, levelVo);
        ServiceResponse<Page<GradeConsumeAreaDTO>> gradeConsumeAreaResponse = consumeStructureApiService.pageGradeConsumeArea(params);
        this.goDownComponent.parseTotal(gradeConsumeAreaResponse, vo);
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel().intValue() == levelVo.getLevel()){
            params.setCountType(3);
            //无归属
            goDownComponent.setValueWgsForArea(params, levelVo);
            ServiceResponse<Page<GradeConsumeAreaDTO>> wgsResponse = consumeStructureApiService.pageGradeConsumeArea(params);
            this.goDownComponent.parseWgs(wgsResponse, vo);
        }
        return RestResponse.success(vo);
    }

    @RequestMapping("get-grade-consume-group")
    public RestResponse queryAgeConsumeShare(GradeConsumeAreaQoAbstract params) {
        log.info("消费构成-等级-等级段参数：{}", JSON.toJSONString(params));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.listGradeConsumeGroup(params));
    }

    @RequestMapping("get-birth-consume-rate")
    public RestResponse getBirthConsumeRate(BirthConsumeRateQoAbstract params) {
        log.info("消费构成-生日会员-占比参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.getBirthConsumeRate(params));
    }

    @RequestMapping("get-birth-consume-trend")
    public RestResponse getBirthConsumeTrend(BirthConsumeRateQoAbstract params) {
        log.info("消费构成-生日会员-趋势参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryBirthConsumeTrend(params));
    }

    @RequestMapping("get-birth-consume-survey")
    public RestResponse getBirthConsumeSurvey(BirthConsumeRateQoAbstract params) {
        log.info("消费构成-生日会员-概况参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryBirthConsumeSurvey(params));
    }

    @RequestMapping("get-birth-consume-area")
    public RestResponse pageWhetherConsumeArea(BirthConsumeAreaQoAbstract params) {
        log.info("消费构成-生日会员-区域参数：{}", JSON.toJSONString(params));
        BaseSalesVo vo = new BaseSalesVo();
        LevelVo levelVo = this.goDownComponent.setValue(params, vo);
        Integer maxLevel = levelVo.getMaxLevel();
        params.setCountType(1);
        ServiceResponse<Page<BirthConsumeAreaDTO>> pageServiceResponse = consumeStructureApiService.pageBirthConsumeArea(params);
        if (!pageServiceResponse.isSuccess()) {
            return RestResponse.failure(pageServiceResponse.getCode(), pageServiceResponse.getMessage());
        }
        vo.setPage(this.goDownComponent.parseData(pageServiceResponse.getResult(), params.getChannel(), params.getEnterpriseId(), params.getLevel(), maxLevel));
        params.setCountType(2);
        goDownComponent.setValueTotalForArea(params, levelVo);
        ServiceResponse<Page<BirthConsumeAreaDTO>> birthConsumeAreaResponse = consumeStructureApiService.pageBirthConsumeArea(params);
        this.goDownComponent.parseTotal(birthConsumeAreaResponse, vo);
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel().intValue() == levelVo.getLevel()){
            params.setCountType(3);
            //无归属
            goDownComponent.setValueWgsForArea(params, levelVo);
            ServiceResponse<Page<BirthConsumeAreaDTO>> wgsResponse = consumeStructureApiService.pageBirthConsumeArea(params);
            this.goDownComponent.parseWgs(wgsResponse, vo);
        }
        return RestResponse.success(vo);
    }

    @RequestMapping("get-old-consume-rate")
    public RestResponse getOldConsumeRate(NewOldConsumeRateQoAbstract params) {
        log.info("消费构成-新老会员-占比参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.getOldConsumeRate(params));
    }

    @RequestMapping("get-old-consume-trend")
    public RestResponse queryOldConsumeTrend(NewOldConsumeRateQoAbstract params) {
        log.info("消费构成-新老会员-趋势参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryOldConsumeTrend(params));
    }

    @RequestMapping("get-old-consume-survey")
    public RestResponse getBirthConsumeSurvey(NewOldConsumeRateQoAbstract params) {
        log.info("消费构成-新老会员-概况参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryNewOldConsumeSurvey(params));
    }

    @RequestMapping("get-old-consume-area")
    public RestResponse queryGradeConsumeArea(NewOldConsumeAreaQoAbstract params) {
        log.info("消费构成-新老会员-区域参数：{}", JSON.toJSONString(params));
        BaseSalesVo vo = new BaseSalesVo();
        LevelVo levelVo = this.goDownComponent.setValue(params, vo);
        Integer maxLevel = levelVo.getMaxLevel();
        params.setCountType(1);
        ServiceResponse<Page<NewOldConsumeAreaDTO>> pageServiceResponse = consumeStructureApiService.pageNewOldConsumeArea(params);
        if (!pageServiceResponse.isSuccess()) {
            return RestResponse.failure(pageServiceResponse.getCode(), pageServiceResponse.getMessage());
        }
        vo.setPage(this.goDownComponent.parseData(pageServiceResponse.getResult(), params.getChannel(), params.getEnterpriseId(), params.getLevel(), maxLevel));
        params.setCountType(2);
        goDownComponent.setValueTotalForArea(params, levelVo);
        ServiceResponse<Page<NewOldConsumeAreaDTO>> newOldConsumeAreaResponse = consumeStructureApiService.pageNewOldConsumeArea(params);
        this.goDownComponent.parseTotal(newOldConsumeAreaResponse, vo);
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel().intValue() == levelVo.getLevel()){
            params.setCountType(3);
            //无归属
            goDownComponent.setValueWgsForArea(params, levelVo);
            ServiceResponse<Page<NewOldConsumeAreaDTO>> wgsResponse = consumeStructureApiService.pageNewOldConsumeArea(params);
            this.goDownComponent.parseWgs(wgsResponse, vo);
        }
        return RestResponse.success(vo);
    }

    @RequestMapping("get-whether-consume-rate")
    public RestResponse getWhetherConsumeRate(WhetherConsumeRateQoAbstract params) {
        log.info("消费构成-是否会员-占比参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.getWhetherConsumeRate(params));
    }

    @RequestMapping("get-whether-consume-trend")
    public RestResponse queryWhetherConsumeTrend(WhetherConsumeRateQoAbstract params) {
        log.info("消费构成-是否会员-趋势参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryWhetherConsumeTrend(params));
    }

    @RequestMapping("get-whether-consume-survey")
    public RestResponse queryWhetherConsumeSurvey(WhetherConsumeRateQoAbstract params) {
        log.info("消费构成-是否会员-概况参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(storeSearchUtils.onLineStoreSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(consumeStructureApiService.queryWhetherConsumeSurvey(params));
    }

    @RequestMapping("get-whether-consume-area")
    public RestResponse pageWhetherConsumeArea(WhetherConsumeAreaQoAbstract params) {
        log.info("消费构成-是否会员-区域参数：{}", JSON.toJSONString(params));
        BaseSalesVo vo = new BaseSalesVo();
        LevelVo levelVo = this.goDownComponent.setValue(params, vo);
        Integer maxLevel = levelVo.getMaxLevel();
        params.setCountType(1);
        ServiceResponse<Page<WhetherConsumeAreaDTO>> pageServiceResponse = consumeStructureApiService.pageWhetherConsumeArea(params);
        if (!pageServiceResponse.isSuccess()) {
            return RestResponse.failure(pageServiceResponse.getCode(), pageServiceResponse.getMessage());
        }
        vo.setPage(this.goDownComponent.parseData(pageServiceResponse.getResult(), params.getChannel(), params.getEnterpriseId(), params.getLevel(), maxLevel));
        params.setCountType(2);
        goDownComponent.setValueTotalForArea(params, levelVo);
        ServiceResponse<Page<WhetherConsumeAreaDTO>> whetherConsumeAreaResponse = consumeStructureApiService.pageWhetherConsumeArea(params);
        this.goDownComponent.parseTotal(whetherConsumeAreaResponse, vo);
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel().intValue() == levelVo.getLevel()){
            //无归属
            params.setCountType(3);
            goDownComponent.setValueWgsForArea(params, levelVo);
            ServiceResponse<Page<WhetherConsumeAreaDTO>> wgsResponse = consumeStructureApiService.pageWhetherConsumeArea(params);
            this.goDownComponent.parseWgs(wgsResponse, vo);
        }
        return RestResponse.success(vo);
    }
}
