package com.gic.cloud.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.UserApiService;
import com.gic.cloud.dto.FunctionDTO;
import com.gic.cloud.dto.FunctionModuleDTO;
import com.gic.cloud.service.FunctionApiService;
import com.gic.cloud.service.FunctionModuleApiService;
import com.gic.cloud.service.IndexApiService;
import com.gic.cloud.web.auth.DataAuthUtils;
import com.gic.cloud.web.constant.ErrorCode;
import com.gic.cloud.web.qo.DateTypeQo;
import com.gic.cloud.web.utils.DataFormatUtils;
import com.gic.cloud.web.vo.FunctionModuleVo;
import com.gic.cloud.web.vo.LevelNameVo;
import com.gic.cloud.web.vo.LevelVo;
import com.gic.cloud.web.vo.MbrVo;
import com.gic.cloud.web.vo.index.IndexConsume;
import com.gic.cloud.web.vo.index.IndexSalesVo;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.data.api.dto.*;
import com.gic.data.api.service.IndexSalesApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 字典
 * @ClassName: IndexController 
 * @Description:  
 * @author zhiwj 
 * @date 2020/9/7 1:35 PM 
 */
@RestController
public class IndexController {
    private Logger log = LogManager.getLogger(IndexController.class);
    @Autowired
    private IndexSalesApiService indexSalesApiService;
    @Autowired
    private StoreSearchUtils storeSearchUtils;
    @Autowired
    private DataAuthUtils dataAuthUtils;
    @Autowired
    private FunctionApiService functionApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private UserApiService userApiService;

    @RequestMapping("index-mbr")
    public RestResponse indexMbr(Integer enterpriseId, Integer userId){
        List<MbrVo> mbrArea = this.dataAuthUtils.getMbrArea(userId, enterpriseId);
        return RestResponse.success(mbrArea);
    }

    @RequestMapping("index-sales")
    public RestResponse indexSales(Integer userId, Integer enterpriseId, String searchJson,
                                   DateTypeQo dateTypeQo, String mbrAreaId, Integer dataType){
        List<Integer> list = this.storeSearchUtils.storeSearch(userId, enterpriseId, searchJson);
        List<String> onLine = this.storeSearchUtils.onLineStoreSearch(userId, enterpriseId, searchJson);
        boolean wgs = storeSearchUtils.isWgs(userId, enterpriseId, searchJson) == 1;
        if(wgs){
            list.add(-1);
            onLine.add("-1");
        }
        ServiceResponse<IndexSalesDTO> indexSales = this.indexSalesApiService.getIndexSales(enterpriseId, list, onLine,
                StringUtils.isNoneBlank(mbrAreaId) ? Long.valueOf(mbrAreaId) : null, dateTypeQo.getType(), dateTypeQo.getDate(), dataType);
        return RestResponse.success(EntityUtil.changeEntityByJSON(IndexSalesVo.class, indexSales.getResult()));
    }

    @RequestMapping("index-member")
    public RestResponse indexMember(Integer userId, Integer enterpriseId, String searchJson,
                                   DateTypeQo dateTypeQo, String mbrAreaId){
        List<Integer> list = this.storeSearchUtils.storeSearch(userId, enterpriseId, searchJson);
        List<String> onLine = this.storeSearchUtils.onLineStoreSearch(userId, enterpriseId, searchJson);
        boolean wgs = storeSearchUtils.isWgs(userId, enterpriseId, searchJson) == 1;
        if(wgs){
            list.add(-1);
            onLine.add("-1");
        }
        List<Long> mbrAreaIdList = this.dataAuthUtils.getMbrAreaId(userId, enterpriseId);
        ServiceResponse<IndexMemberDTO> indexMember = this.indexSalesApiService.getIndexMember(enterpriseId, list, onLine,
                StringUtils.isNoneBlank(mbrAreaId) ? Long.valueOf(mbrAreaId) : null,
                mbrAreaIdList, dateTypeQo.getType(), dateTypeQo.getDate());
        return RestResponse.success(indexMember.getResult());
    }

    @RequestMapping("index-consume")
    public RestResponse indexConsume(Integer userId, Integer enterpriseId, String searchJson,
                                     DateTypeQo dateTypeQo, String mbrAreaId){
        List<Integer> list = this.storeSearchUtils.storeSearch(userId, enterpriseId, searchJson);
        List<String> onLine = this.storeSearchUtils.onLineStoreSearch(userId, enterpriseId, searchJson);
        boolean wgs = storeSearchUtils.isWgs(userId, enterpriseId, searchJson) == 1;
        if(wgs){
            list.add(-1);
            onLine.add("-1");
        }
        //List<Long> mbrAreaIdList = this.dataAuthUtils.getMbrAreaId(userId, enterpriseId);
        ServiceResponse<IndexMemberOrderCntDTO> allMemberOrderCntResponse = this.indexSalesApiService.getIndexMemberOrderCnt(enterpriseId, list, onLine,
                StringUtils.isNoneBlank(mbrAreaId) ? Long.valueOf(mbrAreaId) : null,
                null, dateTypeQo.getType(), dateTypeQo.getDate(), 1);
        ServiceResponse<IndexMemberOrderCntDTO> memberOrderCntResponse = this.indexSalesApiService.getIndexMemberOrderCnt(enterpriseId, list, onLine,
                StringUtils.isNoneBlank(mbrAreaId) ? Long.valueOf(mbrAreaId) : null,
                null, dateTypeQo.getType(), dateTypeQo.getDate(), 2);
        ServiceResponse<IndexMemberOldOrderCnt> oldOrderCnt = this.indexSalesApiService.getIndexMemberOldOrderCnt(enterpriseId, list, onLine,
                StringUtils.isNoneBlank(mbrAreaId) ? Long.valueOf(mbrAreaId) : null, null,
                dateTypeQo.getType(), dateTypeQo.getDate());
        ServiceResponse<IndexMemberBirthCntDTO> birthOrderCnt = this.indexSalesApiService.getIndexMemberBirthOrderCnt(enterpriseId, list, onLine,
                StringUtils.isNoneBlank(mbrAreaId) ? Long.valueOf(mbrAreaId) : null,
                null, dateTypeQo.getType(), dateTypeQo.getDate());
        log.info("allMemberOrderCntResponse:{}", JSON.toJSONString(allMemberOrderCntResponse));
        log.info("memberOrderCntResponse:{}", JSON.toJSONString(memberOrderCntResponse));
        log.info("oldOrderCnt:{}", JSON.toJSONString(oldOrderCnt));
        log.info("birthOrderCnt:{}", JSON.toJSONString(birthOrderCnt));
        IndexConsume consume = new IndexConsume();
        if(allMemberOrderCntResponse.isSuccess() && allMemberOrderCntResponse.getResult() != null && memberOrderCntResponse.getResult() != null){
            IndexMemberOrderCntDTO allMemberOrderCntDto = allMemberOrderCntResponse.getResult();
            IndexMemberOrderCntDTO memberOrderCntDto = memberOrderCntResponse.getResult();
            consume.setMemberRate(DataFormatUtils.rate(memberOrderCntDto.getOrderCnt(), allMemberOrderCntDto.getOrderCnt()));
            String divide = DataFormatUtils.divide(DataFormatUtils.divideDouble(memberOrderCntDto.getOrderCnt(), allMemberOrderCntDto.getOrderCntHb()),
                    DataFormatUtils.divideDouble(memberOrderCntDto.getOrderCnt(), allMemberOrderCntDto.getOrderCntHb()));
            consume.setMemberHb(divide);
        }
        if(oldOrderCnt.isSuccess() && oldOrderCnt.getResult() != null){
            IndexMemberOldOrderCnt indexMemberOldOrderCnt = oldOrderCnt.getResult();
            consume.setOldRate(DataFormatUtils.rate(indexMemberOldOrderCnt.getOldCnt(), indexMemberOldOrderCnt.getAllCnt()));
            Double oldRate = DataFormatUtils.rateDouble(indexMemberOldOrderCnt.getOldCnt(),indexMemberOldOrderCnt.getAllCnt());
            Double hbRate = DataFormatUtils.rateDouble(indexMemberOldOrderCnt.getOldCntHb(), indexMemberOldOrderCnt.getAllCntHb());
            consume.setOldHb(DataFormatUtils.divide(oldRate, hbRate));
        }
        if(birthOrderCnt.isSuccess() && birthOrderCnt.getResult() != null){
            IndexMemberBirthCntDTO birthOrderCntResult = birthOrderCnt.getResult();
            consume.setBirthRate(DataFormatUtils.rate(birthOrderCntResult.getBirthCnt(), birthOrderCntResult.getAllCnt()));
            Double birthRate = DataFormatUtils.rateDouble(birthOrderCntResult.getBirthCnt(), birthOrderCntResult.getAllCnt());
            Double bhRate = DataFormatUtils.rateDouble(birthOrderCntResult.getBirthCntHb(), birthOrderCntResult.getAllCntHb());
            consume.setBirthHb(DataFormatUtils.divide(birthRate, bhRate));
        }
        return RestResponse.success(consume);
    }

    @RequestMapping("function-list")
    public RestResponse functionList(Integer userId) {
        UserDTO userDTO = this.userApiService.getUserInfoById(userId).getResult();
        if (userDTO == null) {
            return RestResponse.failure(ErrorCode.ACCOUNTNOTEXISTS.getCode(), "用户不存在");
        }
        ServiceResponse<List<MenuDTO>> userMenu = this.menuApiService.getUserMenuOfAppNotTree(userDTO.getEnterpriseId(), userId,
                null, getAppId("gyt-app"));
        if (userMenu.isSuccess()) {
            List<MenuDTO> menuList = userMenu.getResult();
            if (CollectionUtils.isNotEmpty(menuList)) {
                List<FunctionModuleVo> list = menuList.stream().filter(e -> e.getMenuCode().length() == 6 && "FUC".equals(e.getMenuCode().substring(0, 3)))
                        .map(e -> new FunctionModuleVo().setFunctionModuleCode(e.getMenuCode()).setFunctionModuleName(e.getMenuName()).setLogo(e.getMenuUrl()))
                        .collect(Collectors.toList());
                return RestResponse.success(list);
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("get-level")
    public RestResponse getLevel(Integer userId, Integer enterpriseId, String searchJson){
        LevelVo level = this.storeSearchUtils.getLevel(userId, enterpriseId, searchJson);
        List<LevelNameVo> list = new ArrayList<>();
        LevelNameVo vo1 = new LevelNameVo(1, "第一层级", 1 >= level.getLevel() && 1 <= level.getMaxLevel() ? 1: 0);
        LevelNameVo vo2 = new LevelNameVo(2, "第二层级", 2 >= level.getLevel() && 2 <= level.getMaxLevel() ? 1: 0);
        LevelNameVo vo3 = new LevelNameVo(3, "第三层级", 3 >= level.getLevel() && 3 <= level.getMaxLevel() ? 1: 0);
        LevelNameVo vo4 = new LevelNameVo(4, "第四层级", 4 >= level.getLevel() && 4 <= level.getMaxLevel() ? 1: 0);
        LevelNameVo vo5 = new LevelNameVo(5, "第五层级", 5 >= level.getLevel() && 5 <= level.getMaxLevel() ? 1: 0);
        LevelNameVo vo6 = new LevelNameVo(6, "第六层级", 6 >= level.getLevel() && 6 <= level.getMaxLevel() ? 1: 0);
        list.add(vo1);
        list.add(vo2);
        list.add(vo3);
        list.add(vo4);
        list.add(vo5);
        list.add(vo6);
        return RestResponse.success(list);
    }

    private String getAppId(String menuCode) {
        //根据menuCode去查询对应的appId
        ServiceResponse<MenuDTO> resp = menuApiService.getMenuByMenuCode(menuCode);
        if (resp.isSuccess()) {
            MenuDTO tempDTO = resp.getResult();
           return tempDTO.getProject();
        }
        return null;
    }

}
