package com.gic.cloud.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.cloud.web.constant.ErrorCode;
import com.gic.cloud.web.vo.CloudDataServiceInfoVO;
import com.gic.cloud.web.vo.EnterpriseVo;
import com.gic.cloud.web.vo.LoginUserVo;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.open.api.dto.EnterpriseServeInfoDTO;
import com.gic.open.api.service.EnterpriseLicenseApiService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
/**
 * 登录
 * @ClassName: LoginController 
 * @Description:  
 * @author boom 
 * @date 2020/9/7 1:32 PM 
 */
@RestController
public class LoginController {
    private static final Logger log = LogManager.getLogger(LoginController.class);

    @Autowired
    private UserApiService userApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private EnterpriseLicenseApiService enterpriseLicenseApiService;

    /**
     * 单域版本代码
     */
    private static final String SINGLE_DOMAIN_CODE = "basic";

    public static String code = "[{\"country\":\"Andorra\",\"cname\":\"安道尔\",\"codename\":\"AD\",\"code\":\"376\"},{\"country\":\"UnitedArabEmirates\",\"cname\":\"阿拉伯联合酋长国\",\"codename\":\"AE\",\"code\":\"971\"},{\"country\":\"Afghanistan\",\"cname\":\"阿富汗\",\"codename\":\"AF\",\"code\":\"93\"},{\"country\":\"AntiguaandBarbuda\",\"cname\":\"安提瓜和巴布达\",\"codename\":\"AG\",\"code\":\"1268\"},{\"country\":\"Anguilla\",\"cname\":\"安圭拉\",\"codename\":\"AI\",\"code\":\"1264\"},{\"country\":\"Albania\",\"cname\":\"阿尔巴尼亚\",\"codename\":\"AL\",\"code\":\"355\"},{\"country\":\"Armenia\",\"cname\":\"亚美尼亚\",\"codename\":\"AM\",\"code\":\"374\"},{\"country\":\"Angola\",\"cname\":\"安哥拉\",\"codename\":\"AO\",\"code\":\"244\"},{\"country\":\"Argentina\",\"cname\":\"阿根廷\",\"codename\":\"AR\",\"code\":\"54\"},{\"country\":\"AmericanSamoa\",\"cname\":\"美属萨摩亚\",\"codename\":\"AS\",\"code\":\"1684\"},{\"country\":\"Austria\",\"cname\":\"奥地利\",\"codename\":\"AT\",\"code\":\"43\"},{\"country\":\"Australia\",\"cname\":\"澳大利亚\",\"codename\":\"AU\",\"code\":\"61\"},{\"country\":\"Aruba\",\"cname\":\"阿鲁巴\",\"codename\":\"AW\",\"code\":\"297\"},{\"country\":\"Azerbaijan\",\"cname\":\"阿塞拜疆\",\"codename\":\"AZ\",\"code\":\"994\"},{\"country\":\"BosniaandHerzegovina\",\"cname\":\"波斯尼亚和黑塞哥维那\",\"codename\":\"BA\",\"code\":\"387\"},{\"country\":\"Barbados\",\"cname\":\"巴巴多斯\",\"codename\":\"BB\",\"code\":\"1246\"},{\"country\":\"Bangladesh\",\"cname\":\"孟加拉国\",\"codename\":\"BD\",\"code\":\"880\"},{\"country\":\"Belgium\",\"cname\":\"比利时\",\"codename\":\"BE\",\"code\":\"32\"},{\"country\":\"BurkinaFaso\",\"cname\":\"布基纳法索\",\"codename\":\"BF\",\"code\":\"226\"},{\"country\":\"Bulgaria\",\"cname\":\"保加利亚\",\"codename\":\"BG\",\"code\":\"359\"},{\"country\":\"Bahrain\",\"cname\":\"巴林\",\"codename\":\"BH\",\"code\":\"973\"},{\"country\":\"Burundi\",\"cname\":\"布隆迪\",\"codename\":\"BI\",\"code\":\"257\"},{\"country\":\"Benin\",\"cname\":\"贝宁\",\"codename\":\"BJ\",\"code\":\"229\"},{\"country\":\"Bermuda\",\"cname\":\"百慕大群岛\",\"codename\":\"BM\",\"code\":\"1441\"},{\"country\":\"Brunei\",\"cname\":\"文莱\",\"codename\":\"BN\",\"code\":\"673\"},{\"country\":\"Bolivia\",\"cname\":\"玻利维亚\",\"codename\":\"BO\",\"code\":\"591\"},{\"country\":\"CaribischNederland\",\"cname\":\"荷兰加勒比\",\"codename\":\"BQ\",\"code\":\"599\"},{\"country\":\"China\",\"cname\":\"中国\",\"codename\":\"ZH\",\"code\":\"86\"},{\"country\":\"Brazil\",\"cname\":\"巴西\",\"codename\":\"BR\",\"code\":\"55\"},{\"country\":\"Bahamas\",\"cname\":\"巴哈马\",\"codename\":\"BS\",\"code\":\"1242\"},{\"country\":\"Bhutan\",\"cname\":\"不丹\",\"codename\":\"BT\",\"code\":\"975\"},{\"country\":\"Botswana\",\"cname\":\"博茨瓦纳\",\"codename\":\"BW\",\"code\":\"267\"},{\"country\":\"Belarus\",\"cname\":\"白俄罗斯\",\"codename\":\"BY\",\"code\":\"375\"},{\"country\":\"Belize\",\"cname\":\"伯利兹\",\"codename\":\"BZ\",\"code\":\"501\"},{\"country\":\"DemocraticRepublicoftheCongo\",\"cname\":\"刚果民主共和国\",\"codename\":\"CD\",\"code\":\"243\"},{\"country\":\"CentralAfricanRepublic\",\"cname\":\"中非共和国\",\"codename\":\"CF\",\"code\":\"236\"},{\"country\":\"RepublicOfTheCongo\",\"cname\":\"刚果共和国\",\"codename\":\"CG\",\"code\":\"242\"},{\"country\":\"Switzerland\",\"cname\":\"瑞士\",\"codename\":\"CH\",\"code\":\"41\"},{\"country\":\"IvoryCoast\",\"cname\":\"象牙海岸\",\"codename\":\"CI\",\"code\":\"225\"},{\"country\":\"CookIslands\",\"cname\":\"库克群岛\",\"codename\":\"CK\",\"code\":\"682\"},{\"country\":\"Chile\",\"cname\":\"智利\",\"codename\":\"CL\",\"code\":\"56\"},{\"country\":\"Cameroon\",\"cname\":\"喀麦隆\",\"codename\":\"CM\",\"code\":\"237\"},{\"country\":\"Colombia\",\"cname\":\"哥伦比亚\",\"codename\":\"CO\",\"code\":\"57\"},{\"country\":\"CostaRica\",\"cname\":\"哥斯达黎加\",\"codename\":\"CR\",\"code\":\"506\"},{\"country\":\"CapeVerde\",\"cname\":\"开普\",\"codename\":\"CV\",\"code\":\"238\"},{\"country\":\"Curacao\",\"cname\":\"库拉索\",\"codename\":\"CW\",\"code\":\"599-9\"},{\"country\":\"Cyprus\",\"cname\":\"塞浦路斯\",\"codename\":\"CY\",\"code\":\"357\"},{\"country\":\"Czech\",\"cname\":\"捷克\",\"codename\":\"CZ\",\"code\":\"420\"},{\"country\":\"Germany\",\"cname\":\"德国\",\"codename\":\"DE\",\"code\":\"49\"},{\"country\":\"Djibouti\",\"cname\":\"吉布提\",\"codename\":\"DJ\",\"code\":\"253\"},{\"country\":\"Denmark\",\"cname\":\"丹麦\",\"codename\":\"DK\",\"code\":\"45\"},{\"country\":\"Dominica\",\"cname\":\"多米尼加\",\"codename\":\"DM\",\"code\":\"1767\"},{\"country\":\"dominicanrepublic\",\"cname\":\"多米尼加共和国\",\"codename\":\"DO\",\"code\":\"1809\"},{\"country\":\"Algeria\",\"cname\":\"阿尔及利亚\",\"codename\":\"DZ\",\"code\":\"213\"},{\"country\":\"Ecuador\",\"cname\":\"厄瓜多尔\",\"codename\":\"EC\",\"code\":\"593\"},{\"country\":\"Estonia\",\"cname\":\"爱沙尼亚\",\"codename\":\"EE\",\"code\":\"372\"},{\"country\":\"Egypt\",\"cname\":\"埃及\",\"codename\":\"EG\",\"code\":\"20\"},{\"country\":\"Eritrea\",\"cname\":\"厄立特里亚\",\"codename\":\"ER\",\"code\":\"291\"},{\"country\":\"Spain\",\"cname\":\"西班牙\",\"codename\":\"ES\",\"code\":\"34\"},{\"country\":\"Ethiopia\",\"cname\":\"埃塞俄比亚\",\"codename\":\"ET\",\"code\":\"251\"},{\"country\":\"Finland\",\"cname\":\"芬兰\",\"codename\":\"FI\",\"code\":\"358\"},{\"country\":\"Fiji\",\"cname\":\"斐济\",\"codename\":\"FJ\",\"code\":\"679\"},{\"country\":\"Micronesia\",\"cname\":\"密克罗尼西亚\",\"codename\":\"FM\",\"code\":\"691\"},{\"country\":\"FaroeIslands\",\"cname\":\"法罗群岛\",\"codename\":\"FO\",\"code\":\"298\"},{\"country\":\"France\",\"cname\":\"法国\",\"codename\":\"FR\",\"code\":\"33\"},{\"country\":\"Gabon\",\"cname\":\"加蓬\",\"codename\":\"GA\",\"code\":\"241\"},{\"country\":\"UnitedKingdom\",\"cname\":\"英国\",\"codename\":\"GB\",\"code\":\"44\"},{\"country\":\"Grenada\",\"cname\":\"格林纳达\",\"codename\":\"GD\",\"code\":\"1473\"},{\"country\":\"Georgia\",\"cname\":\"格鲁吉亚\",\"codename\":\"GE\",\"code\":\"995\"},{\"country\":\"FrenchGuiana\",\"cname\":\"法属圭亚那\",\"codename\":\"GF\",\"code\":\"594\"},{\"country\":\"Ghana\",\"cname\":\"加纳\",\"codename\":\"GH\",\"code\":\"233\"},{\"country\":\"Gibraltar\",\"cname\":\"直布罗陀\",\"codename\":\"GI\",\"code\":\"350\"},{\"country\":\"Greenland\",\"cname\":\"格陵兰岛\",\"codename\":\"GL\",\"code\":\"299\"},{\"country\":\"Gambia\",\"cname\":\"冈比亚\",\"codename\":\"GM\",\"code\":\"220\"},{\"country\":\"Guinea\",\"cname\":\"几内亚\",\"codename\":\"GN\",\"code\":\"224\"},{\"country\":\"Guadeloupe\",\"cname\":\"瓜德罗普岛\",\"codename\":\"GP\",\"code\":\"590\"},{\"country\":\"EquatorialGuinea\",\"cname\":\"赤道几内亚\",\"codename\":\"GQ\",\"code\":\"240\"},{\"country\":\"Greece\",\"cname\":\"希腊\",\"codename\":\"GR\",\"code\":\"30\"},{\"country\":\"Guatemala\",\"cname\":\"瓜地马拉\",\"codename\":\"GT\",\"code\":\"502\"},{\"country\":\"Guam\",\"cname\":\"关岛\",\"codename\":\"GU\",\"code\":\"1671\"},{\"country\":\"Guinea-Bissau\",\"cname\":\"几内亚比绍共和国\",\"codename\":\"GW\",\"code\":\"245\"},{\"country\":\"Guyana\",\"cname\":\"圭亚那\",\"codename\":\"GY\",\"code\":\"592\"},{\"country\":\"HongKong\",\"cname\":\"中国香港\",\"codename\":\"HK\",\"code\":\"852\"},{\"country\":\"Honduras\",\"cname\":\"洪都拉斯\",\"codename\":\"HN\",\"code\":\"504\"},{\"country\":\"Croatia\",\"cname\":\"克罗地亚\",\"codename\":\"HR\",\"code\":\"385\"},{\"country\":\"Haiti\",\"cname\":\"海地\",\"codename\":\"HT\",\"code\":\"509\"},{\"country\":\"Hungary\",\"cname\":\"匈牙利\",\"codename\":\"HU\",\"code\":\"36\"},{\"country\":\"Indonesia\",\"cname\":\"印度尼西亚\",\"codename\":\"ID\",\"code\":\"62\"},{\"country\":\"Ireland\",\"cname\":\"爱尔兰\",\"codename\":\"IE\",\"code\":\"353\"},{\"country\":\"Israel\",\"cname\":\"以色列\",\"codename\":\"IL\",\"code\":\"972\"},{\"country\":\"India\",\"cname\":\"印度\",\"codename\":\"IN\",\"code\":\"91\"},{\"country\":\"Iraq\",\"cname\":\"伊拉克\",\"codename\":\"IQ\",\"code\":\"964\"},{\"country\":\"Iceland\",\"cname\":\"冰岛\",\"codename\":\"IS\",\"code\":\"354\"},{\"country\":\"Italy\",\"cname\":\"意大利\",\"codename\":\"IT\",\"code\":\"39\"},{\"country\":\"Jamaica\",\"cname\":\"牙买加\",\"codename\":\"JM\",\"code\":\"1876\"},{\"country\":\"Jordan\",\"cname\":\"约旦\",\"codename\":\"JO\",\"code\":\"962\"},{\"country\":\"Japan\",\"cname\":\"日本\",\"codename\":\"JP\",\"code\":\"81\"},{\"country\":\"Kenya\",\"cname\":\"肯尼亚\",\"codename\":\"KE\",\"code\":\"254\"},{\"country\":\"Kyrgyzstan\",\"cname\":\"吉尔吉斯斯坦\",\"codename\":\"KG\",\"code\":\"996\"},{\"country\":\"Cambodia\",\"cname\":\"柬埔寨\",\"codename\":\"KH\",\"code\":\"855\"},{\"country\":\"Kiribati\",\"cname\":\"基里巴斯\",\"codename\":\"KI\",\"code\":\"686\"},{\"country\":\"Comoros\",\"cname\":\"科摩罗/马约特\",\"codename\":\"KM\",\"code\":\"269\"},{\"country\":\"SaintKittsandNevis\",\"cname\":\"圣基茨和尼维斯\",\"codename\":\"KN\",\"code\":\"1869\"},{\"country\":\"SouthKorea\",\"cname\":\"韩国\",\"codename\":\"KR\",\"code\":\"82\"},{\"country\":\"Kuwait\",\"cname\":\"科威特\",\"codename\":\"KW\",\"code\":\"965\"},{\"country\":\"CaymanIslands\",\"cname\":\"开曼群岛\",\"codename\":\"KY\",\"code\":\"1345\"},{\"country\":\"Laos\",\"cname\":\"老挝\",\"codename\":\"LA\",\"code\":\"856\"},{\"country\":\"Lebanon\",\"cname\":\"黎巴嫩\",\"codename\":\"LB\",\"code\":\"961\"},{\"country\":\"SaintLucia\",\"cname\":\"圣露西亚\",\"codename\":\"LC\",\"code\":\"1758\"},{\"country\":\"Liechtenstein\",\"cname\":\"列支敦士登\",\"codename\":\"LI\",\"code\":\"423\"},{\"country\":\"SriLanka\",\"cname\":\"斯里兰卡\",\"codename\":\"LK\",\"code\":\"94\"},{\"country\":\"Liberia\",\"cname\":\"利比里亚\",\"codename\":\"LR\",\"code\":\"231\"},{\"country\":\"Lesotho\",\"cname\":\"莱索托\",\"codename\":\"LS\",\"code\":\"266\"},{\"country\":\"Lithuania\",\"cname\":\"立陶宛\",\"codename\":\"LT\",\"code\":\"370\"},{\"country\":\"Luxembourg\",\"cname\":\"卢森堡\",\"codename\":\"LU\",\"code\":\"352\"},{\"country\":\"Latvia\",\"cname\":\"拉脱维亚\",\"codename\":\"LV\",\"code\":\"371\"},{\"country\":\"Libya\",\"cname\":\"利比亚\",\"codename\":\"LY\",\"code\":\"218\"},{\"country\":\"Morocco\",\"cname\":\"摩洛哥\",\"codename\":\"MA\",\"code\":\"212\"},{\"country\":\"Monaco\",\"cname\":\"摩纳哥\",\"codename\":\"MC\",\"code\":\"377\"},{\"country\":\"Moldova\",\"cname\":\"摩尔多瓦\",\"codename\":\"MD\",\"code\":\"373\"},{\"country\":\"Montenegro\",\"cname\":\"黑山\",\"codename\":\"ME\",\"code\":\"382\"},{\"country\":\"Madagascar\",\"cname\":\"马达加斯加\",\"codename\":\"MG\",\"code\":\"261\"},{\"country\":\"MarshallIslands\",\"cname\":\"马绍尔群岛\",\"codename\":\"MH\",\"code\":\"692\"},{\"country\":\"Macedonia\",\"cname\":\"马其顿\",\"codename\":\"MK\",\"code\":\"389\"},{\"country\":\"Mali\",\"cname\":\"马里\",\"codename\":\"ML\",\"code\":\"223\"},{\"country\":\"Myanmar\",\"cname\":\"缅甸\",\"codename\":\"MM\",\"code\":\"95\"},{\"country\":\"Mongolia\",\"cname\":\"蒙古\",\"codename\":\"MN\",\"code\":\"976\"},{\"country\":\"Macau\",\"cname\":\"中国澳门\",\"codename\":\"MO\",\"code\":\"853\"},{\"country\":\"Mauritania\",\"cname\":\"毛里塔尼亚\",\"codename\":\"MR\",\"code\":\"222\"},{\"country\":\"Montserrat\",\"cname\":\"蒙特塞拉特岛\",\"codename\":\"MS\",\"code\":\"1664\"},{\"country\":\"Malta\",\"cname\":\"马耳他\",\"codename\":\"MT\",\"code\":\"356\"},{\"country\":\"Mauritius\",\"cname\":\"毛里求斯\",\"codename\":\"MU\",\"code\":\"230\"},{\"country\":\"Maldives\",\"cname\":\"马尔代夫\",\"codename\":\"MV\",\"code\":\"960\"},{\"country\":\"Malawi\",\"cname\":\"马拉维\",\"codename\":\"MW\",\"code\":\"265\"},{\"country\":\"Mexico\",\"cname\":\"墨西哥\",\"codename\":\"MX\",\"code\":\"52\"},{\"country\":\"Malaysia\",\"cname\":\"马来西亚\",\"codename\":\"MY\",\"code\":\"60\"},{\"country\":\"Mozambique\",\"cname\":\"莫桑比克\",\"codename\":\"MZ\",\"code\":\"258\"},{\"country\":\"Namibia\",\"cname\":\"纳米比亚\",\"codename\":\"NA\",\"code\":\"264\"},{\"country\":\"NewCaledonia\",\"cname\":\"新喀里多尼亚\",\"codename\":\"NC\",\"code\":\"687\"},{\"country\":\"Niger\",\"cname\":\"尼日尔\",\"codename\":\"NE\",\"code\":\"227\"},{\"country\":\"Nigeria\",\"cname\":\"尼日利亚\",\"codename\":\"NG\",\"code\":\"234\"},{\"country\":\"Nicaragua\",\"cname\":\"尼加拉瓜\",\"codename\":\"NI\",\"code\":\"505\"},{\"country\":\"Netherlands\",\"cname\":\"荷兰\",\"codename\":\"NL\",\"code\":\"31\"},{\"country\":\"Norway\",\"cname\":\"挪威\",\"codename\":\"NO\",\"code\":\"47\"},{\"country\":\"Nepal\",\"cname\":\"尼泊尔\",\"codename\":\"NP\",\"code\":\"977\"},{\"country\":\"Nauru\",\"cname\":\"拿鲁岛\",\"codename\":\"NR\",\"code\":\"674\"},{\"country\":\"NewZealand\",\"cname\":\"新西兰\",\"codename\":\"NZ\",\"code\":\"64\"},{\"country\":\"Oman\",\"cname\":\"阿曼\",\"codename\":\"OM\",\"code\":\"968\"},{\"country\":\"Panama\",\"cname\":\"巴拿马\",\"codename\":\"PA\",\"code\":\"507\"},{\"country\":\"Peru\",\"cname\":\"秘鲁\",\"codename\":\"PE\",\"code\":\"51\"},{\"country\":\"FrenchPolynesia\",\"cname\":\"法属波利尼西亚\",\"codename\":\"PF\",\"code\":\"689\"},{\"country\":\"PapuaNewGuinea\",\"cname\":\"巴布亚新几内亚\",\"codename\":\"PG\",\"code\":\"675\"},{\"country\":\"Philippines\",\"cname\":\"菲律宾\",\"codename\":\"PH\",\"code\":\"63\"},{\"country\":\"Pakistan\",\"cname\":\"巴基斯坦\",\"codename\":\"PK\",\"code\":\"92\"},{\"country\":\"Poland\",\"cname\":\"波兰\",\"codename\":\"PL\",\"code\":\"48\"},{\"country\":\"SaintPierreandMiquelon\",\"cname\":\"圣彼埃尔和密克隆岛\",\"codename\":\"PM\",\"code\":\"508\"},{\"country\":\"PuertoRico\",\"cname\":\"波多黎各\",\"codename\":\"PR\",\"code\":\"1787\"},{\"country\":\"Portugal\",\"cname\":\"葡萄牙\",\"codename\":\"PT\",\"code\":\"351\"},{\"country\":\"Palau\",\"cname\":\"帕劳\",\"codename\":\"PW\",\"code\":\"680\"},{\"country\":\"Paraguay\",\"cname\":\"巴拉圭\",\"codename\":\"PY\",\"code\":\"595\"},{\"country\":\"Qatar\",\"cname\":\"卡塔尔\",\"codename\":\"QA\",\"code\":\"974\"},{\"country\":\"RéunionIsland\",\"cname\":\"留尼汪\",\"codename\":\"RE\",\"code\":\"262\"},{\"country\":\"Romania\",\"cname\":\"罗马尼亚\",\"codename\":\"RO\",\"code\":\"40\"},{\"country\":\"Serbia\",\"cname\":\"塞尔维亚\",\"codename\":\"RS\",\"code\":\"381\"},{\"country\":\"Russia\",\"cname\":\"俄罗斯/哈萨克斯坦\",\"codename\":\"RU\",\"code\":\"7\"},{\"country\":\"Rwanda\",\"cname\":\"卢旺达\",\"codename\":\"RW\",\"code\":\"250\"},{\"country\":\"SaudiArabia\",\"cname\":\"沙特阿拉伯\",\"codename\":\"SA\",\"code\":\"966\"},{\"country\":\"SolomonIslands\",\"cname\":\"所罗门群岛\",\"codename\":\"SB\",\"code\":\"677\"},{\"country\":\"Seychelles\",\"cname\":\"塞舌尔\",\"codename\":\"SC\",\"code\":\"248\"},{\"country\":\"Sudan\",\"cname\":\"苏丹\",\"codename\":\"SD\",\"code\":\"249\"},{\"country\":\"Sweden\",\"cname\":\"瑞典\",\"codename\":\"SE\",\"code\":\"46\"},{\"country\":\"Singapore\",\"cname\":\"新加坡\",\"codename\":\"SG\",\"code\":\"65\"},{\"country\":\"Slovenia\",\"cname\":\"斯洛文尼亚\",\"codename\":\"SI\",\"code\":\"386\"},{\"country\":\"Slovakia\",\"cname\":\"斯洛伐克\",\"codename\":\"SK\",\"code\":\"421\"},{\"country\":\"SierraLeone\",\"cname\":\"塞拉利昂\",\"codename\":\"SL\",\"code\":\"232\"},{\"country\":\"SanMarino\",\"cname\":\"圣马力诺\",\"codename\":\"SM\",\"code\":\"378\"},{\"country\":\"Senegal\",\"cname\":\"塞内加尔\",\"codename\":\"SN\",\"code\":\"221\"},{\"country\":\"Somalia\",\"cname\":\"索马里\",\"codename\":\"SO\",\"code\":\"252\"},{\"country\":\"Suriname\",\"cname\":\"苏里南\",\"codename\":\"SR\",\"code\":\"597\"},{\"country\":\"SaoTomeandPrincipe\",\"cname\":\"圣多美和普林西比\",\"codename\":\"ST\",\"code\":\"239\"},{\"country\":\"ElSalvador\",\"cname\":\"萨尔瓦多\",\"codename\":\"SV\",\"code\":\"503\"},{\"country\":\"Swaziland\",\"cname\":\"斯威士兰\",\"codename\":\"SZ\",\"code\":\"268\"},{\"country\":\"TurksandCaicosIslands\",\"cname\":\"特克斯和凯科斯群岛\",\"codename\":\"TC\",\"code\":\"1649\"},{\"country\":\"Chad\",\"cname\":\"乍得\",\"codename\":\"TD\",\"code\":\"235\"},{\"country\":\"Togo\",\"cname\":\"多哥\",\"codename\":\"TG\",\"code\":\"228\"},{\"country\":\"Thailand\",\"cname\":\"泰国\",\"codename\":\"TH\",\"code\":\"66\"},{\"country\":\"Tajikistan\",\"cname\":\"塔吉克斯坦\",\"codename\":\"TJ\",\"code\":\"992\"},{\"country\":\"EastTimor\",\"cname\":\"东帝汶\",\"codename\":\"TL\",\"code\":\"670\"},{\"country\":\"Turkmenistan\",\"cname\":\"土库曼斯坦\",\"codename\":\"TM\",\"code\":\"993\"},{\"country\":\"Tunisia\",\"cname\":\"突尼斯\",\"codename\":\"TN\",\"code\":\"216\"},{\"country\":\"Tonga\",\"cname\":\"汤加\",\"codename\":\"TO\",\"code\":\"676\"},{\"country\":\"Turkey\",\"cname\":\"土耳其\",\"codename\":\"TR\",\"code\":\"90\"},{\"country\":\"TrinidadandTobago\",\"cname\":\"特立尼达和多巴哥\",\"codename\":\"TT\",\"code\":\"1868\"},{\"country\":\"Taiwan\",\"cname\":\"中国台湾\",\"codename\":\"TW\",\"code\":\"886\"},{\"country\":\"Tanzania\",\"cname\":\"坦桑尼亚\",\"codename\":\"TZ\",\"code\":\"255\"},{\"country\":\"Ukraine\",\"cname\":\"乌克兰\",\"codename\":\"UA\",\"code\":\"380\"},{\"country\":\"Uganda\",\"cname\":\"乌干达\",\"codename\":\"UG\",\"code\":\"256\"},{\"country\":\"UnitedStates\",\"cname\":\"美国\",\"codename\":\"US\",\"code\":\"1\"},{\"country\":\"Uruguay\",\"cname\":\"乌拉圭\",\"codename\":\"UY\",\"code\":\"598\"},{\"country\":\"Uzbekistan\",\"cname\":\"乌兹别克斯坦\",\"codename\":\"UZ\",\"code\":\"998\"},{\"country\":\"SaintVincentandTheGrenadines\",\"cname\":\"圣文森特和格林纳丁斯\",\"codename\":\"VC\",\"code\":\"1784\"},{\"country\":\"Venezuela\",\"cname\":\"委内瑞拉\",\"codename\":\"VE\",\"code\":\"58\"},{\"country\":\"VirginIslands,British\",\"cname\":\"英属处女群岛\",\"codename\":\"VG\",\"code\":\"1284\"},{\"country\":\"Vietnam\",\"cname\":\"越南\",\"codename\":\"VN\",\"code\":\"84\"},{\"country\":\"Vanuatu\",\"cname\":\"瓦努阿图\",\"codename\":\"VU\",\"code\":\"678\"},{\"country\":\"Samoa\",\"cname\":\"萨摩亚\",\"codename\":\"WS\",\"code\":\"685\"},{\"country\":\"Yemen\",\"cname\":\"也门\",\"codename\":\"YE\",\"code\":\"967\"},{\"country\":\"SouthAfrica\",\"cname\":\"南非\",\"codename\":\"ZA\",\"code\":\"27\"},{\"country\":\"Zambia\",\"cname\":\"赞比亚\",\"codename\":\"ZM\",\"code\":\"260\"},{\"country\":\"Zimbabwe\",\"cname\":\"津巴布韦\",\"codename\":\"ZW\",\"code\":\"263\"}]";


    @RequestMapping("send-code")
    public RestResponse sendCode(String phoneNumber, String areaCode){
        String smsSenceId = "GYT0001";
        String tsPhone = "18000000000";
        String code = "1234";
        List<UserDTO> userDTOList = this.userApiService.listUserByPhoneNumber(phoneNumber, "+" + areaCode).getResult();
        if(CollectionUtils.isNotEmpty(userDTOList)){
            //过滤没有观云台应用的用户
            userDTOList = userDTOList.stream().filter(userDTO -> checkCloudAppExist(userDTO.getEnterpriseId())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(userDTOList)) {
                return RestResponse.failure(ErrorCode.ACCOUNTNOTEXISTS.getCode(), "观云台账号不存在");
            }
            List<Integer> enterpriseIdList = userDTOList.stream()
                    .map(userDTO -> userDTO.getEnterpriseId()).collect(Collectors.toList());
            List<EnterpriseDTO> enterpriseDTOList = this.enterpriseApiService.listEnterpriseByIds(enterpriseIdList).getResult();
            //随机取一个用户
            Integer enterpriseId = userDTOList.get(0).getEnterpriseId();
            ServiceResponse<Void> smsSendResult = ServiceResponse.success();
            if(!tsPhone.equals(phoneNumber)){
                code = CreateRandomUtils.generateNumberStr(4);
                smsSendResult = smsSendApiService.sendPlatformSms(smsSenceId,enterpriseId,areaCode, phoneNumber, new String[]{code});
            }
            if(smsSendResult.isSuccess()){
                String key = "GYTCode:" + areaCode + ":" + phoneNumber;
                LoginUserVo userVo = new LoginUserVo();
                userVo.setEnterpriseId(enterpriseId);
                userVo.setAreaCode(areaCode);
                userVo.setCode(code);
                userVo.setPhoneNumber(phoneNumber);
                userVo.setUserId(userDTOList.get(0).getUserId());
                userVo.setUserName(userDTOList.get(0).getUserName());

                enterpriseDTOList.stream().forEach(enterpriseDTO -> {
                    if(enterpriseDTO.getEnterpriseId().intValue() == enterpriseId.intValue()){
                        userVo.setCreateTime(enterpriseDTO.getCreateTime());
                        userVo.setAuditTime(enterpriseDTO.getAuditTime());
                        if(phoneNumber.equals(enterpriseDTO.getPhoneNumber())){
                            userVo.setIsAdmin(1);
                        }
                        if(SINGLE_DOMAIN_CODE.equals(enterpriseDTO.getVersionCode())){
                            userVo.setIsSingle(1);
                        }else {
                            userVo.setIsSingle(0);
                        }
                    }
                });
                RedisUtil.setCache(key, userVo, 5L, TimeUnit.MINUTES);
                return RestResponse.success(enterpriseId);
            }else {
                return RestResponse.failure(ErrorCode.SYSTEMERROR.getCode(), smsSendResult.getMessage());
            }
        }else {
            return RestResponse.failure(ErrorCode.ACCOUNTNOTEXISTS.getCode(), ErrorCode.ACCOUNTNOTEXISTS.getMsg());
        }
    }

    @RequestMapping("login")
    public RestResponse login(String areaCode, String phoneNumber, String code){
        String key = "GYTCode:" + areaCode + ":" + phoneNumber;
        Object cache = RedisUtil.getCache(key);
        if(cache != null){
            LoginUserVo userVo = (LoginUserVo) cache;
            if(userVo.getCode().equals(code)){
                return RestResponse.success(userVo);
            }else {
                return RestResponse.failure(ErrorCode.SYSTEMERROR.getCode(), "验证码错误");
            }
        }else {
            return RestResponse.failure(ErrorCode.ACCOUNTNOTEXISTS.getCode(), "请发送验证码");
        }
    }

    @RequestMapping("all-enterprise")
    public RestResponse allEnterprise(Integer userId){
        UserDTO userDTO = this.userApiService.getUserInfoById(userId).getResult();
        if(userDTO == null){
            return RestResponse.success();
        }
        List<UserDTO> userDTOList = this.userApiService.listUserByPhoneNumber(userDTO.getPhoneNumber(), userDTO.getPhoneAreaCode()).getResult();
        log.info("userDTOList:{}", JSON.toJSONString(userDTOList));
        if(CollectionUtils.isNotEmpty(userDTOList)){
            List<Integer> enterpriseIdList = userDTOList.stream()
                    .filter(user -> checkCloudAppExist(user.getEnterpriseId()))
                    .map(user -> user.getEnterpriseId()).collect(Collectors.toList());
            List<EnterpriseDTO> enterpriseDTOList = this.enterpriseApiService.listEnterpriseByIds(enterpriseIdList).getResult();
            List<EnterpriseVo> result = new ArrayList<>();
            if(CollectionUtils.isNotEmpty(enterpriseDTOList)){
                result = enterpriseDTOList.stream().map(enterpriseDTO -> {
                    EnterpriseVo vo = EntityUtil.changeEntityByJSON(EnterpriseVo.class, enterpriseDTO);
                    vo.setIsSingle(SINGLE_DOMAIN_CODE.equals(enterpriseDTO.getVersionCode()) ? 1 : 0);
                    if(CollectionUtils.isNotEmpty(userDTOList)){
                        userDTOList.stream().forEach(user -> {
                            if(user.getEnterpriseId().intValue() == enterpriseDTO.getEnterpriseId().intValue()){
                                vo.setIsAdmin((user.getPhoneNumber().equals(enterpriseDTO.getPhoneNumber())) ? 1 : 0);
                                vo.setUserName(user.getUserName());
                                vo.setUserId(user.getUserId());
                            }
                        });
                    }
                    return vo;
                }).collect(Collectors.toList());
            }
            return RestResponse.success(result);
        }
        return RestResponse.success();
    }

    @RequestMapping("nation-code")
    public RestResponse getNationcode(){
        return RestResponse.success(JSON.parseArray(code));
    }

    /**
     * 服务到期时间提醒
     * @param enterpriseId
     * @return
     */
    @RequestMapping("/service-expire-time")
    public RestResponse license(Integer enterpriseId) {
        ServiceResponse<EnterpriseServeInfoDTO> info = enterpriseLicenseApiService
                .getServeInfo(enterpriseId, "cloud-data");
        return ResultControllerUtils.commonResultOne(info, CloudDataServiceInfoVO.class);
    }

    @RequestMapping("/enterprise-service-info")
    public RestResponse enterpriseServiceExpireTime(Integer enterpriseId) {
        ServiceResponse<EnterpriseDTO> info = enterpriseApiService
                .getEnterpriseById(enterpriseId);
        return ResultControllerUtils.commonResultOne(info, EnterpriseVo.class);
    }

    private boolean checkCloudAppExist(Integer enterpriseId) {
        ServiceResponse<EnterpriseServeInfoDTO> info = enterpriseLicenseApiService
                .getServeInfo(enterpriseId, "cloud-data");
        if (info.isSuccess() && info.getResult() != null && info.getResult().getServeStatus() != 1) {
            return true;
        }
        return false;
    }
}
