package com.gic.cloud.web.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.EnterprisePortraitRelDTO;
import com.gic.cloud.service.EnterprisePortraitRelApiService;
import com.gic.cloud.web.auth.DataAuthUtils;
import com.gic.cloud.web.vo.BaseSalesVo;
import com.gic.cloud.web.vo.LevelVo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.data.api.dto.member.MemberCustomPortraitDTO;
import com.gic.data.api.dto.member.MemberDistributionAreaDTO;
import com.gic.data.api.dto.member.MemberRecruitAreaDTO;
import com.gic.data.api.qo.member.*;
import com.gic.data.api.service.member.MemberCloudDataApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
/**
 * 会员数据
 * @ClassName: MemberCloudDataController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/7 1:38 PM 
 */
@RestController
public class MemberCloudDataController {
    private static final Logger log = LogManager.getLogger(MemberCloudDataController.class);
    @Autowired
    private MemberCloudDataApiService memberCloudDataApiService;
    @Autowired
    private StoreSearchUtils storeSearchUtils;
    @Autowired
    private DataAuthUtils dataAuthUtils;
    @Autowired
    private EnterprisePortraitRelApiService enterprisePortraitRelApiService;
    @Autowired
    private GoDownComponent goDownComponent;

    /**
     * 会员分布-趋势
     * @Title: queryMemberDistributionTrend 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("get-member-distribution-trend")
    public RestResponse queryMemberDistributionTrend(MemberDistributionTrendQoAbstract params) {
        log.info("会员分布-趋势参数：{}", JSON.toJSONString(params));
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        params.setOnlineStoreIdList(new ArrayList<>());
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(memberCloudDataApiService.queryMemberDistributionTrend(params));
    }

    @RequestMapping("get-member-distribution-area")
    public RestResponse pageMemberDistributionArea(MemberDistributionAreaQoAbstract params) {
        log.info("会员分布-区域参数：{}", JSON.toJSONString(params));
        //线下
        params.setChannel(1);

        params.setOnlineStoreIdList(new ArrayList<>());

        BaseSalesVo vo = new BaseSalesVo();
        LevelVo levelVo = this.goDownComponent.setValue(params, vo);
        Integer maxLevel = levelVo.getMaxLevel();
        params.setCountType(1);
        ServiceResponse<Page<MemberDistributionAreaDTO>> pageServiceResponse = memberCloudDataApiService.pageMemberDistributionArea(params);
        if (!pageServiceResponse.isSuccess()) {
            return RestResponse.failure(pageServiceResponse.getCode(), pageServiceResponse.getMessage());
        }
        vo.setPage(this.goDownComponent.parseData(pageServiceResponse.getResult(), params.getChannel(), params.getEnterpriseId(), params.getLevel(), maxLevel));
        params.setCountType(2);
        goDownComponent.setValueTotalForArea(params, levelVo);
        ServiceResponse<Page<MemberDistributionAreaDTO>> memberDistributionAreaResponse = memberCloudDataApiService.pageMemberDistributionArea(params);
        this.goDownComponent.parseTotal(memberDistributionAreaResponse, vo);
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel() == levelVo.getLevel().intValue()){
            //无归属
            params.setCountType(3);
            goDownComponent.setValueWgsForArea(params, levelVo);
            ServiceResponse<Page<MemberDistributionAreaDTO>> wgsResponse = memberCloudDataApiService.pageMemberDistributionArea(params);
            this.goDownComponent.parseWgs(wgsResponse, vo);
        }
        return RestResponse.success(vo);
    }

    @RequestMapping("get-member-recruit-area")
    public RestResponse pageMemberRecruitArea(MemberRecruitAreaQoAbstract params) {
        log.info("会员招募-区域参数：{}", JSON.toJSONString(params));
        //线下
        params.setChannel(1);
        params.setOnlineStoreIdList(new ArrayList<>());

        BaseSalesVo vo = new BaseSalesVo();
        LevelVo levelVo = this.goDownComponent.setValue(params, vo);
        Integer maxLevel = levelVo.getMaxLevel();
        params.setCountType(1);
        ServiceResponse<Page<MemberRecruitAreaDTO>> pageServiceResponse = memberCloudDataApiService.pageMemberRecruitArea(params);
        if (!pageServiceResponse.isSuccess()) {
            return RestResponse.failure(pageServiceResponse.getCode(), pageServiceResponse.getMessage());
        }
        vo.setPage(this.goDownComponent.parseData(pageServiceResponse.getResult(), params.getChannel(), params.getEnterpriseId(), params.getLevel(), maxLevel));
        params.setCountType(2);
        goDownComponent.setValueTotalForArea(params, levelVo);
        ServiceResponse<Page<MemberRecruitAreaDTO>> memberRecruitAreaResponse = memberCloudDataApiService.pageMemberRecruitArea(params);
        this.goDownComponent.parseTotal(memberRecruitAreaResponse, vo);
        if(params.getChannel() != null && params.getChannel() == 1 && params.getLevel() == levelVo.getLevel().intValue()){
            //无归属
            params.setCountType(3);
            goDownComponent.setValueWgsForArea(params, levelVo);
            ServiceResponse<Page<MemberRecruitAreaDTO>> wgsResponse = memberCloudDataApiService.pageMemberRecruitArea(params);
            this.goDownComponent.parseWgs(wgsResponse, vo);
        }
        return RestResponse.success(vo);
    }

    @RequestMapping("get-member-portrait")
    public RestResponse queryMemberBasicPortrait(MemberPortraitQoAbstract params) {
        log.info("会员基础画像参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(new ArrayList<>());
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(memberCloudDataApiService.queryMemberBasicPortrait(params));
    }

    @RequestMapping("get-member-portrait-overview")
    public RestResponse getMemberPortraitOverview(MemberPortraitQoAbstract params) {
        log.info("会员画像-概况参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(new ArrayList<>());
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(memberCloudDataApiService.getMemberPortraitOverview(params));
    }

    @RequestMapping("get-member-custom-portrait")
    public RestResponse queryMemberCustomPortrait(MemberCustomPortraitQoAbstract params) {
        log.info("会员自定义画像参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(new ArrayList<>());
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(memberCloudDataApiService.queryMemberCustomPortrait(params));
    }

    @RequestMapping("get-member-recruit-overview")
    public RestResponse getMemberRecruitOverview(MemberOverviewQoAbstract params) {
        log.info("会员概况-会员招募概况参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(new ArrayList<>());
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(memberCloudDataApiService.getMemberRecruitOverview(params));
    }

    @RequestMapping("get-member-recruit-trend")
    public RestResponse queryMemberRecruitTrend(MemberOverviewQoAbstract params) {
        log.info("会员概况-会员招募趋势参数：{}", JSON.toJSONString(params));
        params.setOnlineStoreIdList(new ArrayList<>());
        params.setStoreInfoIdList(storeSearchUtils.storeSearch(params.getUserId(), params.getEnterpriseId(), params.getSearchJson()));
        params.setMemberCardIdList(dataAuthUtils.getMbrAreaId(params.getUserId(), params.getEnterpriseId()));
        goDownComponent.setValueTotal(params);
        return ResultControllerUtils.commonResult(memberCloudDataApiService.queryMemberRecruitTrend(params));
    }

    /**
     * 移除恢复
     * @Title: updateMemberPortraitFieldStatus 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param userId
     * @param fieldCode
     * @param dataType 1：基础画像 2：自定义
     * @param status  0：放到回收站 1：恢复
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("update-member-portrait-field-status")
    public RestResponse updateMemberPortraitFieldStatus(Integer enterpriseId, Integer userId, String fieldCode, Integer dataType, Integer status) {
        return ResultControllerUtils.commonResult(enterprisePortraitRelApiService.updateStatus(enterpriseId, fieldCode, dataType, status));
    }

    /**
     * 排序
     * @Title: sort 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param userId
     * @param fieldCode
     * @param dataType 1：基础画像 2：自定义
     * @param sort 要排序的位置👌
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("sort-member-portrait-field")
    public RestResponse sort(Integer enterpriseId, Integer userId, String fieldCode, Integer dataType, Integer sort) {
        return ResultControllerUtils.commonResult(enterprisePortraitRelApiService.sort(enterpriseId, fieldCode, dataType, sort));
    }


    @RequestMapping("get-member-custom-portrait-field")
    public RestResponse getMemberCustomPortraitField(MemberCustomPortraitQoAbstract params) {
        log.info("会员自定义画像-字段筛选参数：{}", JSON.toJSONString(params));
        Integer enterpriseId = params.getEnterpriseId();
        ServiceResponse<List<MemberCustomPortraitDTO>> fieldListRes = memberCloudDataApiService.listCustomField(params);
        if (fieldListRes.isSuccess()) {
            List<MemberCustomPortraitDTO> fieldList = fieldListRes.getResult();
            if (CollectionUtils.isNotEmpty(fieldList)) {
                //最新的字段列表
                Set<String> lastFieldSet = fieldList.stream().map(e -> e.getFieldCode()).collect(Collectors.toSet());
                for (MemberCustomPortraitDTO temp : fieldList) {
                    enterprisePortraitRelApiService.savePortraitField(enterpriseId, temp.getFieldCode(), temp.getFieldName(), 2);
                }
                ServiceResponse<List<EnterprisePortraitRelDTO>> portraitFieldRes = enterprisePortraitRelApiService.listPortraitField(enterpriseId,  2, params.getDataStatus());
                //同步删除
                if (portraitFieldRes.isSuccess()) {
                    List<EnterprisePortraitRelDTO> currentPortraitFieldList = portraitFieldRes.getResult();
                    if (CollectionUtils.isNotEmpty(currentPortraitFieldList)) {
                        currentPortraitFieldList.forEach(e -> {
                            boolean hasNotExist = !lastFieldSet.contains(e.getPortraitCode());
                            if (hasNotExist) {
                                //删除
                                enterprisePortraitRelApiService.delete(e.getPortraitRelationId());
                            }
                        });
                    }
                    return RestResponse.success(currentPortraitFieldList.stream().filter(e -> lastFieldSet.contains(e.getPortraitCode())).collect(Collectors.toList()));
                } else {
                    return RestResponse.failure(portraitFieldRes.getCode(), portraitFieldRes.getMessage());
                }
            }
        }
        return RestResponse.failure(fieldListRes.getCode(), fieldListRes.getMessage());
    }

    @RequestMapping("get-member-basic-portrait-field")
    public RestResponse getMemberBasicPortraitField(MemberCustomPortraitQoAbstract params) {
        log.info("会员基础画像-字段筛选参数：{}", JSON.toJSONString(params));
        Integer enterpriseId = params.getEnterpriseId();
        List<EnterprisePortraitRelDTO> list = new ArrayList<>(6);
        list.add(new EnterprisePortraitRelDTO()
                .setCreateTime(new Date())
                .setUpdateTime(new Date())
                .setDataType(1)
                .setEnterpriseId(enterpriseId)
                .setPortraitCode("1")
                .setSort(1)
                .setStatus(1)
                .setDeleteFlag(0)
                .setPortraitName("会员等级"));
        list.add(new EnterprisePortraitRelDTO()
                .setCreateTime(new Date())
                .setUpdateTime(new Date())
                .setDataType(1)
                .setEnterpriseId(enterpriseId)
                .setPortraitCode("2")
                .setSort(2)
                .setStatus(1)
                .setDeleteFlag(0)
                .setPortraitName("性别"));
        list.add(new EnterprisePortraitRelDTO()
                .setCreateTime(new Date())
                .setUpdateTime(new Date())
                .setDataType(1)
                .setEnterpriseId(enterpriseId)
                .setPortraitCode("3")
                .setSort(3)
                .setStatus(1)
                .setDeleteFlag(0)
                .setPortraitName("年龄段"));
        list.add(new EnterprisePortraitRelDTO()
                .setCreateTime(new Date())
                .setUpdateTime(new Date())
                .setDataType(1)
                .setEnterpriseId(enterpriseId)
                .setPortraitCode("4")
                .setSort(4)
                .setStatus(1)
                .setDeleteFlag(0)
                .setPortraitName("4开卡渠道"));
        list.add(new EnterprisePortraitRelDTO()
                .setCreateTime(new Date())
                .setUpdateTime(new Date())
                .setDataType(1)
                .setEnterpriseId(enterpriseId)
                .setPortraitCode("5")
                .setSort(5)
                .setStatus(1)
                .setDeleteFlag(0)
                .setPortraitName("生日月份"));
        list.add(new EnterprisePortraitRelDTO()
                .setCreateTime(new Date())
                .setUpdateTime(new Date())
                .setDataType(1)
                .setEnterpriseId(enterpriseId)
                .setPortraitCode("6")
                .setSort(6)
                .setStatus(1)
                .setDeleteFlag(0)
                .setPortraitName("星座名称"));
        enterprisePortraitRelApiService.insertForeach(list);
        ServiceResponse<List<EnterprisePortraitRelDTO>> portraitFieldRes = enterprisePortraitRelApiService.listPortraitField(enterpriseId,  1, params.getDataStatus());
        return ResultControllerUtils.commonResult(portraitFieldRes);
    }


}
